/*
    Copyright (C) 2006 by OXIM TEAM

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "oximint.h"

typedef struct {
    char source[4];
    char target[4];
} ChineseCharacter;

typedef struct {
    const ChineseCharacter *elements;
    unsigned int           total;
} ChineseCharMaps;

static const ChineseCharacter big5_elements[] = {
    {"¯", "ˉ"},
    {"·", "・"},
    {"Π", "∏"},
    {"Σ", "∑"},
    {"‥", "¨"},
    {"‧", "・"},
    {"‵", "｀"},
    {"∥", "‖"},
    {"≒", "≈"},
    {"≦", "≤"},
    {"≧", "≥"},
    {"─", "―"},
    {"╱", "／"},
    {"╲", "＼"},
    {"╴", "＿"},
    {"〝", "＂"},
    {"〞", "″"},
    {"丟", "丢"},
    {"並", "并"},
    {"丼", "井"},
    {"乾", "干"},
    {"亂", "乱"},
    {"亙", "亘"},
    {"亞", "亚"},
    {"伕", "夫"},
    {"佇", "伫"},
    {"佈", "布"},
    {"佔", "占"},
    {"佪", "徊"},
    {"併", "并"},
    {"來", "来"},
    {"侖", "仑"},
    {"侚", "徇"},
    {"侶", "侣"},
    {"侷", "局"},
    {"俁", "俣"},
    {"係", "系"},
    {"俠", "侠"},
    {"倀", "伥"},
    {"倆", "俩"},
    {"倉", "仓"},
    {"個", "个"},
    {"們", "们"},
    {"倖", "幸"},
    {"倣", "仿"},
    {"倫", "伦"},
    {"偉", "伟"},
    {"偪", "逼"},
    {"側", "侧"},
    {"偵", "侦"},
    {"偺", "咱"},
    {"偽", "伪"},
    {"傑", "杰"},
    {"傖", "伧"},
    {"傘", "伞"},
    {"備", "备"},
    {"傚", "效"},
    {"傢", "家"},
    {"傭", "佣"},
    {"傯", "偬"},
    {"傳", "传"},
    {"傴", "伛"},
    {"債", "债"},
    {"傷", "伤"},
    {"傾", "倾"},
    {"僂", "偻"},
    {"僅", "仅"},
    {"僉", "佥"},
    {"僊", "仙"},
    {"僑", "侨"},
    {"僕", "仆"},
    {"僣", "僭"},
    {"僥", "侥"},
    {"僨", "偾"},
    {"僱", "雇"},
    {"價", "价"},
    {"儀", "仪"},
    {"儂", "侬"},
    {"億", "亿"},
    {"儅", "当"},
    {"儈", "侩"},
    {"儉", "俭"},
    {"儐", "傧"},
    {"儔", "俦"},
    {"儕", "侪"},
    {"儘", "尽"},
    {"償", "偿"},
    {"優", "优"},
    {"儲", "储"},
    {"儷", "俪"},
    {"儸", "罗"},
    {"儺", "傩"},
    {"儻", "傥"},
    {"儼", "俨"},
    {"兇", "凶"},
    {"兌", "兑"},
    {"兒", "儿"},
    {"兗", "兖"},
    {"內", "内"},
    {"兩", "两"},
    {"冊", "册"},
    {"冑", "胄"},
    {"冪", "幂"},
    {"凅", "涸"},
    {"凈", "净"},
    {"凍", "冻"},
    {"凜", "凛"},
    {"凱", "凯"},
    {"別", "别"},
    {"刪", "删"},
    {"剄", "刭"},
    {"則", "则"},
    {"剉", "锉"},
    {"剋", "克"},
    {"剎", "刹"},
    {"剛", "刚"},
    {"剝", "剥"},
    {"剮", "剐"},
    {"剴", "剀"},
    {"創", "创"},
    {"剷", "铲"},
    {"劃", "划"},
    {"劄", "札"},
    {"劇", "剧"},
    {"劉", "刘"},
    {"劊", "刽"},
    {"劌", "刿"},
    {"劍", "剑"},
    {"劑", "剂"},
    {"劻", "匡"},
    {"勁", "劲"},
    {"動", "动"},
    {"勗", "勖"},
    {"務", "务"},
    {"勛", "勋"},
    {"勝", "胜"},
    {"勞", "劳"},
    {"勢", "势"},
    {"勣", "绩"},
    {"勦", "剿"},
    {"勱", "劢"},
    {"勳", "勋"},
    {"勵", "励"},
    {"勸", "劝"},
    {"勻", "匀"},
    {"匋", "陶"},
    {"匭", "匦"},
    {"匯", "汇"},
    {"匱", "匮"},
    {"區", "区"},
    {"卄", "廿"},
    {"協", "协"},
    {"卬", "昂"},
    {"卹", "恤"},
    {"卻", "却"},
    {"厙", "厍"},
    {"厭", "厌"},
    {"厲", "厉"},
    {"厴", "厣"},
    {"參", "参"},
    {"叡", "睿"},
    {"叢", "丛"},
    {"吋", "寸"},
    {"吳", "吴"},
    {"吶", "呐"},
    {"呂", "吕"},
    {"呎", "尺"},
    {"咷", "啕"},
    {"咼", "呙"},
    {"員", "员"},
    {"唄", "呗"},
    {"唸", "念"},
    {"問", "问"},
    {"啗", "啖"},
    {"啞", "哑"},
    {"啟", "启"},
    {"啣", "衔"},
    {"喚", "唤"},
    {"喪", "丧"},
    {"喫", "吃"},
    {"喬", "乔"},
    {"單", "单"},
    {"喲", "哟"},
    {"嗆", "呛"},
    {"嗇", "啬"},
    {"嗎", "吗"},
    {"嗚", "呜"},
    {"嗩", "唢"},
    {"嗶", "哔"},
    {"嘆", "叹"},
    {"嘍", "喽"},
    {"嘔", "呕"},
    {"嘖", "啧"},
    {"嘗", "尝"},
    {"嘜", "唛"},
    {"嘩", "哗"},
    {"嘮", "唠"},
    {"嘯", "啸"},
    {"嘰", "叽"},
    {"嘵", "哓"},
    {"嘸", "呒"},
    {"噁", "恶"},
    {"噓", "嘘"},
    {"噠", "哒"},
    {"噥", "哝"},
    {"噦", "哕"},
    {"噯", "嗳"},
    {"噲", "哙"},
    {"噴", "喷"},
    {"噸", "吨"},
    {"噹", "当"},
    {"嚀", "咛"},
    {"嚇", "吓"},
    {"嚌", "哜"},
    {"嚐", "尝"},
    {"嚕", "噜"},
    {"嚙", "啮"},
    {"嚥", "咽"},
    {"嚦", "呖"},
    {"嚨", "咙"},
    {"嚮", "向"},
    {"嚳", "喾"},
    {"嚴", "严"},
    {"嚶", "嘤"},
    {"囀", "啭"},
    {"囁", "嗫"},
    {"囂", "嚣"},
    {"囅", "冁"},
    {"囈", "呓"},
    {"囉", "罗"},
    {"囌", "苏"},
    {"囑", "嘱"},
    {"囓", "啮"},
    {"囪", "囱"},
    {"圇", "囵"},
    {"國", "国"},
    {"圍", "围"},
    {"園", "园"},
    {"圓", "圆"},
    {"圖", "图"},
    {"團", "团"},
    {"坵", "丘"},
    {"埜", "野"},
    {"埡", "垭"},
    {"執", "执"},
    {"埼", "崎"},
    {"堅", "坚"},
    {"堊", "垩"},
    {"堝", "埚"},
    {"堯", "尧"},
    {"報", "报"},
    {"場", "场"},
    {"堿", "碱"},
    {"塊", "块"},
    {"塋", "茔"},
    {"塏", "垲"},
    {"塒", "埘"},
    {"塗", "涂"},
    {"塚", "冢"},
    {"塢", "坞"},
    {"塤", "埙"},
    {"塵", "尘"},
    {"塹", "堑"},
    {"墊", "垫"},
    {"墑", "墒"},
    {"墜", "坠"},
    {"墫", "樽"},
    {"墮", "堕"},
    {"墳", "坟"},
    {"墻", "墙"},
    {"墾", "垦"},
    {"壇", "坛"},
    {"壎", "埙"},
    {"壓", "压"},
    {"壘", "垒"},
    {"壙", "圹"},
    {"壚", "垆"},
    {"壞", "坏"},
    {"壟", "垄"},
    {"壢", "坜"},
    {"壩", "坝"},
    {"壯", "壮"},
    {"壺", "壶"},
    {"壽", "寿"},
    {"夠", "够"},
    {"夢", "梦"},
    {"夥", "伙"},
    {"夾", "夹"},
    {"奐", "奂"},
    {"奧", "奥"},
    {"奩", "奁"},
    {"奪", "夺"},
    {"奮", "奋"},
    {"妝", "妆"},
    {"妳", "你"},
    {"姍", "姗"},
    {"姦", "奸"},
    {"姪", "侄"},
    {"娛", "娱"},
    {"婁", "娄"},
    {"婦", "妇"},
    {"婬", "淫"},
    {"婭", "娅"},
    {"媧", "娲"},
    {"媮", "偷"},
    {"媯", "妫"},
    {"媼", "媪"},
    {"媽", "妈"},
    {"媿", "愧"},
    {"嫋", "袅"},
    {"嫗", "妪"},
    {"嫵", "妩"},
    {"嫻", "娴"},
    {"嬈", "娆"},
    {"嬋", "婵"},
    {"嬌", "娇"},
    {"嬙", "嫱"},
    {"嬝", "袅"},
    {"嬡", "嫒"},
    {"嬤", "嬷"},
    {"嬪", "嫔"},
    {"嬭", "奶"},
    {"嬰", "婴"},
    {"嬸", "婶"},
    {"孃", "娘"},
    {"孌", "娈"},
    {"孫", "孙"},
    {"學", "学"},
    {"孿", "孪"},
    {"宮", "宫"},
    {"寘", "置"},
    {"寢", "寝"},
    {"實", "实"},
    {"寧", "宁"},
    {"審", "审"},
    {"寫", "写"},
    {"寬", "宽"},
    {"寵", "宠"},
    {"寶", "宝"},
    {"將", "将"},
    {"專", "专"},
    {"尋", "寻"},
    {"對", "对"},
    {"導", "导"},
    {"尷", "尴"},
    {"屆", "届"},
    {"屍", "尸"},
    {"屜", "屉"},
    {"屝", "扉"},
    {"屢", "屡"},
    {"層", "层"},
    {"屨", "屦"},
    {"屬", "属"},
    {"岡", "冈"},
    {"峴", "岘"},
    {"島", "岛"},
    {"峽", "峡"},
    {"崍", "崃"},
    {"崑", "昆"},
    {"崗", "岗"},
    {"崙", "仑"},
    {"崠", "岽"},
    {"崢", "峥"},
    {"崳", "嵛"},
    {"嵐", "岚"},
    {"嵒", "岩"},
    {"嶁", "嵝"},
    {"嶄", "崭"},
    {"嶇", "岖"},
    {"嶗", "崂"},
    {"嶠", "峤"},
    {"嶧", "峄"},
    {"嶸", "嵘"},
    {"嶺", "岭"},
    {"嶼", "屿"},
    {"嶽", "岳"},
    {"巋", "岿"},
    {"巒", "峦"},
    {"巔", "巅"},
    {"巖", "岩"},
    {"巰", "巯"},
    {"巹", "卺"},
    {"帥", "帅"},
    {"師", "师"},
    {"帳", "帐"},
    {"帶", "带"},
    {"幀", "帧"},
    {"幃", "帏"},
    {"幗", "帼"},
    {"幘", "帻"},
    {"幟", "帜"},
    {"幣", "币"},
    {"幫", "帮"},
    {"幬", "帱"},
    {"幵", "开"},
    {"幹", "干"},
    {"幾", "几"},
    {"庂", "仄"},
    {"庫", "库"},
    {"廁", "厕"},
    {"廂", "厢"},
    {"廄", "厩"},
    {"廈", "厦"},
    {"廚", "厨"},
    {"廝", "厮"},
    {"廟", "庙"},
    {"廠", "厂"},
    {"廡", "庑"},
    {"廢", "废"},
    {"廣", "广"},
    {"廩", "廪"},
    {"廬", "庐"},
    {"廱", "痈"},
    {"廳", "厅"},
    {"弒", "弑"},
    {"弔", "吊"},
    {"弳", "弪"},
    {"張", "张"},
    {"強", "强"},
    {"彆", "别"},
    {"彈", "弹"},
    {"彊", "强"},
    {"彌", "弥"},
    {"彎", "弯"},
    {"彙", "汇"},
    {"彥", "彦"},
    {"彫", "雕"},
    {"彿", "佛"},
    {"後", "后"},
    {"徑", "径"},
    {"從", "从"},
    {"徠", "徕"},
    {"復", "复"},
    {"徬", "旁"},
    {"徹", "彻"},
    {"恆", "恒"},
    {"恥", "耻"},
    {"悅", "悦"},
    {"悵", "怅"},
    {"悶", "闷"},
    {"悽", "凄"},
    {"惇", "敦"},
    {"惡", "恶"},
    {"惱", "恼"},
    {"惲", "恽"},
    {"惷", "蠢"},
    {"惻", "恻"},
    {"愛", "爱"},
    {"愜", "惬"},
    {"愨", "悫"},
    {"愴", "怆"},
    {"愷", "恺"},
    {"愾", "忾"},
    {"慄", "栗"},
    {"慇", "殷"},
    {"態", "态"},
    {"慍", "愠"},
    {"慘", "惨"},
    {"慚", "惭"},
    {"慟", "恸"},
    {"慣", "惯"},
    {"慪", "怄"},
    {"慫", "怂"},
    {"慮", "虑"},
    {"慳", "悭"},
    {"慶", "庆"},
    {"慼", "戚"},
    {"慾", "欲"},
    {"憂", "忧"},
    {"憊", "惫"},
    {"憐", "怜"},
    {"憑", "凭"},
    {"憒", "愦"},
    {"憚", "惮"},
    {"憤", "愤"},
    {"憫", "悯"},
    {"憮", "怃"},
    {"憲", "宪"},
    {"憶", "忆"},
    {"懃", "勤"},
    {"懇", "恳"},
    {"應", "应"},
    {"懌", "怿"},
    {"懍", "懔"},
    {"懞", "蒙"},
    {"懟", "怼"},
    {"懣", "懑"},
    {"懨", "恹"},
    {"懲", "惩"},
    {"懶", "懒"},
    {"懷", "怀"},
    {"懸", "悬"},
    {"懺", "忏"},
    {"懼", "惧"},
    {"懾", "慑"},
    {"戀", "恋"},
    {"戇", "戆"},
    {"戉", "钺"},
    {"戔", "戋"},
    {"戧", "戗"},
    {"戩", "戬"},
    {"戰", "战"},
    {"戲", "戏"},
    {"戶", "户"},
    {"扐", "仂"},
    {"扞", "捍"},
    {"扱", "插"},
    {"扺", "抵"},
    {"抃", "拚"},
    {"抔", "抱"},
    {"抴", "曳"},
    {"拋", "抛"},
    {"拑", "钳"},
    {"挌", "格"},
    {"挶", "局"},
    {"挾", "挟"},
    {"捨", "舍"},
    {"捫", "扪"},
    {"捲", "卷"},
    {"掃", "扫"},
    {"掄", "抡"},
    {"掙", "挣"},
    {"掛", "挂"},
    {"採", "采"},
    {"揀", "拣"},
    {"揚", "扬"},
    {"換", "换"},
    {"揮", "挥"},
    {"揹", "背"},
    {"搆", "构"},
    {"損", "损"},
    {"搖", "摇"},
    {"搗", "捣"},
    {"搟", "擀"},
    {"搥", "捶"},
    {"搨", "打"},
    {"搯", "掏"},
    {"搶", "抢"},
    {"搾", "榨"},
    {"摀", "捂"},
    {"摃", "扛"},
    {"摑", "掴"},
    {"摜", "掼"},
    {"摟", "搂"},
    {"摯", "挚"},
    {"摳", "抠"},
    {"摶", "抟"},
    {"摻", "掺"},
    {"撈", "捞"},
    {"撐", "撑"},
    {"撓", "挠"},
    {"撚", "捻"},
    {"撟", "挢"},
    {"撢", "掸"},
    {"撣", "掸"},
    {"撥", "拨"},
    {"撦", "扯"},
    {"撫", "抚"},
    {"撲", "扑"},
    {"撳", "揿"},
    {"撻", "挞"},
    {"撾", "挝"},
    {"撿", "捡"},
    {"擁", "拥"},
    {"擄", "掳"},
    {"擇", "择"},
    {"擊", "击"},
    {"擋", "挡"},
    {"擔", "担"},
    {"據", "据"},
    {"擠", "挤"},
    {"擣", "捣"},
    {"擬", "拟"},
    {"擯", "摈"},
    {"擰", "拧"},
    {"擱", "搁"},
    {"擲", "掷"},
    {"擴", "扩"},
    {"擷", "撷"},
    {"擺", "摆"},
    {"擻", "擞"},
    {"擼", "撸"},
    {"擾", "扰"},
    {"攄", "摅"},
    {"攆", "撵"},
    {"攏", "拢"},
    {"攔", "拦"},
    {"攖", "撄"},
    {"攙", "搀"},
    {"攛", "撺"},
    {"攜", "携"},
    {"攝", "摄"},
    {"攢", "攒"},
    {"攣", "挛"},
    {"攤", "摊"},
    {"攪", "搅"},
    {"攬", "揽"},
    {"攷", "考"},
    {"敗", "败"},
    {"敘", "叙"},
    {"敵", "敌"},
    {"數", "数"},
    {"斂", "敛"},
    {"斃", "毙"},
    {"斕", "斓"},
    {"斬", "斩"},
    {"斷", "断"},
    {"旂", "旗"},
    {"旛", "幡"},
    {"昇", "升"},
    {"時", "时"},
    {"晉", "晋"},
    {"晝", "昼"},
    {"晞", "曦"},
    {"晢", "晰"},
    {"晻", "暗"},
    {"暈", "晕"},
    {"暉", "晖"},
    {"暘", "阳"},
    {"暢", "畅"},
    {"暫", "暂"},
    {"暱", "昵"},
    {"暸", "了"},
    {"曄", "晔"},
    {"曆", "历"},
    {"曇", "昙"},
    {"曉", "晓"},
    {"曏", "向"},
    {"曖", "暧"},
    {"曠", "旷"},
    {"曬", "晒"},
    {"書", "书"},
    {"會", "会"},
    {"朢", "望"},
    {"朧", "胧"},
    {"朮", "术"},
    {"杇", "圬"},
    {"東", "东"},
    {"枴", "拐"},
    {"柵", "栅"},
    {"柺", "拐"},
    {"栒", "旬"},
    {"桮", "杯"},
    {"桿", "杆"},
    {"梔", "栀"},
    {"條", "条"},
    {"梟", "枭"},
    {"梱", "捆"},
    {"棄", "弃"},
    {"棖", "枨"},
    {"棗", "枣"},
    {"棟", "栋"},
    {"棧", "栈"},
    {"棲", "栖"},
    {"椏", "桠"},
    {"楄", "匾"},
    {"楊", "杨"},
    {"楓", "枫"},
    {"楙", "茂"},
    {"楜", "胡"},
    {"楨", "桢"},
    {"業", "业"},
    {"極", "极"},
    {"榦", "干"},
    {"榪", "杩"},
    {"榮", "荣"},
    {"榿", "桤"},
    {"槃", "盘"},
    {"構", "构"},
    {"槍", "枪"},
    {"槓", "杠"},
    {"槧", "椠"},
    {"槨", "椁"},
    {"槳", "桨"},
    {"樁", "桩"},
    {"樂", "乐"},
    {"樅", "枞"},
    {"樑", "梁"},
    {"樓", "楼"},
    {"標", "标"},
    {"樞", "枢"},
    {"樣", "样"},
    {"樸", "朴"},
    {"樹", "树"},
    {"樺", "桦"},
    {"橈", "桡"},
    {"橋", "桥"},
    {"機", "机"},
    {"橢", "椭"},
    {"橦", "幢"},
    {"橫", "横"},
    {"檁", "檩"},
    {"檉", "柽"},
    {"檔", "档"},
    {"檜", "桧"},
    {"檢", "检"},
    {"檣", "樯"},
    {"檯", "台"},
    {"檳", "槟"},
    {"檸", "柠"},
    {"檻", "槛"},
    {"櫂", "棹"},
    {"櫃", "柜"},
    {"櫐", "累"},
    {"櫓", "橹"},
    {"櫚", "榈"},
    {"櫛", "栉"},
    {"櫝", "椟"},
    {"櫞", "橼"},
    {"櫟", "栎"},
    {"櫥", "橱"},
    {"櫧", "槠"},
    {"櫨", "栌"},
    {"櫪", "枥"},
    {"櫫", "橥"},
    {"櫬", "榇"},
    {"櫳", "栊"},
    {"櫸", "榉"},
    {"櫺", "棂"},
    {"櫻", "樱"},
    {"欄", "栏"},
    {"權", "权"},
    {"欏", "椤"},
    {"欒", "栾"},
    {"欖", "榄"},
    {"欞", "棂"},
    {"欸", "唉"},
    {"欽", "钦"},
    {"歎", "叹"},
    {"歐", "欧"},
    {"歟", "欤"},
    {"歡", "欢"},
    {"歲", "岁"},
    {"歷", "历"},
    {"歸", "归"},
    {"歿", "殁"},
    {"殀", "夭"},
    {"殘", "残"},
    {"殞", "殒"},
    {"殤", "殇"},
    {"殫", "殚"},
    {"殭", "僵"},
    {"殮", "殓"},
    {"殯", "殡"},
    {"殲", "歼"},
    {"殺", "杀"},
    {"殼", "壳"},
    {"殽", "肴"},
    {"毀", "毁"},
    {"毆", "殴"},
    {"毌", "毋"},
    {"毘", "毗"},
    {"毬", "球"},
    {"毿", "毵"},
    {"氈", "毡"},
    {"氌", "氇"},
    {"氣", "气"},
    {"氫", "氢"},
    {"氬", "氩"},
    {"氳", "氲"},
    {"氾", "泛"},
    {"汆", "氽"},
    {"汍", "丸"},
    {"汎", "泛"},
    {"汙", "污"},
    {"決", "决"},
    {"沍", "冱"},
    {"沒", "没"},
    {"沖", "冲"},
    {"況", "况"},
    {"泝", "溯"},
    {"洟", "涕"},
    {"洩", "泄"},
    {"洶", "汹"},
    {"浬", "里"},
    {"浹", "浃"},
    {"涇", "泾"},
    {"涼", "凉"},
    {"淒", "凄"},
    {"淚", "泪"},
    {"淥", "渌"},
    {"淨", "净"},
    {"淪", "沦"},
    {"淵", "渊"},
    {"淶", "涞"},
    {"淺", "浅"},
    {"渙", "涣"},
    {"減", "减"},
    {"渦", "涡"},
    {"測", "测"},
    {"渾", "浑"},
    {"湊", "凑"},
    {"湞", "浈"},
    {"湣", "闵"},
    {"湧", "涌"},
    {"湯", "汤"},
    {"溈", "沩"},
    {"準", "准"},
    {"溝", "沟"},
    {"溫", "温"},
    {"溼", "湿"},
    {"滄", "沧"},
    {"滅", "灭"},
    {"滌", "涤"},
    {"滎", "荥"},
    {"滬", "沪"},
    {"滯", "滞"},
    {"滲", "渗"},
    {"滷", "卤"},
    {"滸", "浒"},
    {"滾", "滚"},
    {"滿", "满"},
    {"漁", "渔"},
    {"漚", "沤"},
    {"漢", "汉"},
    {"漣", "涟"},
    {"漬", "渍"},
    {"漲", "涨"},
    {"漵", "溆"},
    {"漸", "渐"},
    {"漿", "浆"},
    {"潁", "颍"},
    {"潑", "泼"},
    {"潔", "洁"},
    {"潛", "潜"},
    {"潟", "舄"},
    {"潤", "润"},
    {"潯", "浔"},
    {"潰", "溃"},
    {"潷", "滗"},
    {"潿", "涠"},
    {"澀", "涩"},
    {"澂", "澄"},
    {"澆", "浇"},
    {"澇", "涝"},
    {"澔", "浩"},
    {"澗", "涧"},
    {"澠", "渑"},
    {"澤", "泽"},
    {"澩", "泶"},
    {"澮", "浍"},
    {"澱", "淀"},
    {"濁", "浊"},
    {"濃", "浓"},
    {"濕", "湿"},
    {"濘", "泞"},
    {"濛", "蒙"},
    {"濟", "济"},
    {"濤", "涛"},
    {"濫", "滥"},
    {"濬", "浚"},
    {"濰", "潍"},
    {"濱", "滨"},
    {"濺", "溅"},
    {"濼", "泺"},
    {"濾", "滤"},
    {"瀁", "漾"},
    {"瀅", "滢"},
    {"瀆", "渎"},
    {"瀉", "泻"},
    {"瀋", "沈"},
    {"瀏", "浏"},
    {"瀕", "濒"},
    {"瀘", "泸"},
    {"瀝", "沥"},
    {"瀟", "潇"},
    {"瀠", "潆"},
    {"瀦", "潴"},
    {"瀧", "泷"},
    {"瀨", "濑"},
    {"瀰", "弥"},
    {"瀲", "潋"},
    {"瀾", "澜"},
    {"灃", "沣"},
    {"灄", "滠"},
    {"灑", "洒"},
    {"灕", "漓"},
    {"灘", "滩"},
    {"灝", "灏"},
    {"灣", "湾"},
    {"灤", "滦"},
    {"灩", "滟"},
    {"災", "灾"},
    {"炤", "照"},
    {"炰", "炮"},
    {"為", "为"},
    {"烏", "乌"},
    {"烴", "烃"},
    {"無", "无"},
    {"煉", "炼"},
    {"煒", "炜"},
    {"煖", "暖"},
    {"煙", "烟"},
    {"煢", "茕"},
    {"煥", "焕"},
    {"煩", "烦"},
    {"煬", "炀"},
    {"熒", "荧"},
    {"熗", "炝"},
    {"熱", "热"},
    {"熾", "炽"},
    {"燁", "烨"},
    {"燄", "焰"},
    {"燈", "灯"},
    {"燉", "炖"},
    {"燐", "磷"},
    {"燒", "烧"},
    {"燙", "烫"},
    {"燜", "焖"},
    {"營", "营"},
    {"燦", "灿"},
    {"燬", "毁"},
    {"燭", "烛"},
    {"燴", "烩"},
    {"燻", "熏"},
    {"燼", "烬"},
    {"燾", "焘"},
    {"燿", "耀"},
    {"爍", "烁"},
    {"爐", "炉"},
    {"爛", "烂"},
    {"爭", "争"},
    {"爺", "爷"},
    {"爾", "尔"},
    {"牆", "墙"},
    {"牘", "牍"},
    {"牠", "它"},
    {"牴", "抵"},
    {"牽", "牵"},
    {"犖", "荦"},
    {"犛", "牦"},
    {"犢", "犊"},
    {"犧", "牺"},
    {"狀", "状"},
    {"狚", "旦"},
    {"狹", "狭"},
    {"狽", "狈"},
    {"猙", "狰"},
    {"猶", "犹"},
    {"猻", "狲"},
    {"獃", "呆"},
    {"獄", "狱"},
    {"獅", "狮"},
    {"獎", "奖"},
    {"獨", "独"},
    {"獪", "狯"},
    {"獫", "猃"},
    {"獰", "狞"},
    {"獲", "获"},
    {"獵", "猎"},
    {"獷", "犷"},
    {"獸", "兽"},
    {"獺", "獭"},
    {"獻", "献"},
    {"獼", "猕"},
    {"玀", "猡"},
    {"玅", "妙"},
    {"玆", "兹"},
    {"玨", "珏"},
    {"珪", "圭"},
    {"珮", "佩"},
    {"現", "现"},
    {"琱", "雕"},
    {"琺", "珐"},
    {"琿", "珲"},
    {"瑋", "玮"},
    {"瑣", "琐"},
    {"瑤", "瑶"},
    {"瑩", "莹"},
    {"瑪", "玛"},
    {"瑯", "琅"},
    {"璉", "琏"},
    {"璣", "玑"},
    {"璦", "瑷"},
    {"環", "环"},
    {"璽", "玺"},
    {"璿", "璇"},
    {"瓊", "琼"},
    {"瓏", "珑"},
    {"瓔", "璎"},
    {"瓖", "镶"},
    {"瓚", "瓒"},
    {"甌", "瓯"},
    {"甕", "瓮"},
    {"產", "产"},
    {"甦", "苏"},
    {"甪", "角"},
    {"甯", "宁"},
    {"畝", "亩"},
    {"畢", "毕"},
    {"畫", "画"},
    {"畬", "畲"},
    {"異", "异"},
    {"當", "当"},
    {"疇", "畴"},
    {"疊", "叠"},
    {"疿", "痱"},
    {"痙", "痉"},
    {"痠", "酸"},
    {"痲", "麻"},
    {"痳", "麻"},
    {"痺", "痹"},
    {"痾", "疴"},
    {"瘉", "愈"},
    {"瘋", "疯"},
    {"瘍", "疡"},
    {"瘓", "痪"},
    {"瘞", "瘗"},
    {"瘡", "疮"},
    {"瘧", "疟"},
    {"瘺", "瘘"},
    {"療", "疗"},
    {"癆", "痨"},
    {"癇", "痫"},
    {"癉", "瘅"},
    {"癒", "愈"},
    {"癘", "疠"},
    {"癟", "瘪"},
    {"癡", "痴"},
    {"癢", "痒"},
    {"癤", "疖"},
    {"癥", "症"},
    {"癩", "癞"},
    {"癬", "癣"},
    {"癭", "瘿"},
    {"癮", "瘾"},
    {"癰", "痈"},
    {"癱", "瘫"},
    {"癲", "癫"},
    {"發", "发"},
    {"皁", "皂"},
    {"皚", "皑"},
    {"皰", "疱"},
    {"皸", "皲"},
    {"皺", "皱"},
    {"盃", "杯"},
    {"盜", "盗"},
    {"盞", "盏"},
    {"盡", "尽"},
    {"監", "监"},
    {"盤", "盘"},
    {"盧", "卢"},
    {"盪", "荡"},
    {"眥", "眦"},
    {"眾", "众"},
    {"睏", "困"},
    {"睜", "睁"},
    {"睞", "睐"},
    {"睪", "睾"},
    {"瞇", "眯"},
    {"瞞", "瞒"},
    {"瞭", "了"},
    {"瞼", "睑"},
    {"矇", "蒙"},
    {"矓", "胧"},
    {"矚", "瞩"},
    {"矯", "矫"},
    {"砲", "炮"},
    {"硃", "朱"},
    {"硤", "硖"},
    {"硨", "砗"},
    {"硯", "砚"},
    {"碁", "棋"},
    {"碕", "崎"},
    {"碩", "硕"},
    {"碪", "砧"},
    {"碭", "砀"},
    {"確", "确"},
    {"碼", "码"},
    {"磚", "砖"},
    {"磣", "碜"},
    {"磧", "碛"},
    {"磯", "矶"},
    {"磽", "硗"},
    {"礎", "础"},
    {"礙", "碍"},
    {"礡", "礴"},
    {"礦", "矿"},
    {"礪", "砺"},
    {"礫", "砾"},
    {"礬", "矾"},
    {"礱", "砻"},
    {"祂", "他"},
    {"祅", "祆"},
    {"祇", "只"},
    {"祐", "佑"},
    {"祕", "秘"},
    {"祼", "裸"},
    {"祿", "禄"},
    {"禍", "祸"},
    {"禎", "祯"},
    {"禦", "御"},
    {"禪", "禅"},
    {"禮", "礼"},
    {"禰", "祢"},
    {"禱", "祷"},
    {"禿", "秃"},
    {"秈", "籼"},
    {"秏", "耗"},
    {"稅", "税"},
    {"稈", "秆"},
    {"稜", "棱"},
    {"稟", "禀"},
    {"稨", "扁"},
    {"種", "种"},
    {"稱", "称"},
    {"穀", "谷"},
    {"穌", "稣"},
    {"積", "积"},
    {"穎", "颖"},
    {"穡", "穑"},
    {"穢", "秽"},
    {"穨", "颓"},
    {"穩", "稳"},
    {"穫", "获"},
    {"窩", "窝"},
    {"窪", "洼"},
    {"窮", "穷"},
    {"窯", "窑"},
    {"窶", "窭"},
    {"窺", "窥"},
    {"竄", "窜"},
    {"竅", "窍"},
    {"竇", "窦"},
    {"竊", "窃"},
    {"競", "竞"},
    {"笻", "筇"},
    {"筆", "笔"},
    {"筍", "笋"},
    {"筧", "笕"},
    {"筴", "策"},
    {"箄", "箅"},
    {"箇", "个"},
    {"箋", "笺"},
    {"箏", "筝"},
    {"箠", "棰"},
    {"節", "节"},
    {"範", "范"},
    {"築", "筑"},
    {"篋", "箧"},
    {"篛", "箬"},
    {"篠", "筱"},
    {"篤", "笃"},
    {"篩", "筛"},
    {"篲", "彗"},
    {"篳", "筚"},
    {"簀", "箦"},
    {"簍", "篓"},
    {"簑", "蓑"},
    {"簞", "箪"},
    {"簡", "简"},
    {"簣", "篑"},
    {"簫", "箫"},
    {"簷", "檐"},
    {"簽", "签"},
    {"簾", "帘"},
    {"籃", "篮"},
    {"籌", "筹"},
    {"籐", "藤"},
    {"籜", "箨"},
    {"籟", "籁"},
    {"籠", "笼"},
    {"籤", "签"},
    {"籥", "龠"},
    {"籩", "笾"},
    {"籪", "簖"},
    {"籬", "篱"},
    {"籮", "箩"},
    {"籲", "吁"},
    {"粧", "妆"},
    {"粵", "粤"},
    {"糝", "糁"},
    {"糞", "粪"},
    {"糢", "馍"},
    {"糧", "粮"},
    {"糰", "团"},
    {"糲", "粝"},
    {"糴", "籴"},
    {"糶", "粜"},
    {"糾", "纠"},
    {"紀", "纪"},
    {"紂", "纣"},
    {"約", "约"},
    {"紅", "红"},
    {"紆", "纡"},
    {"紇", "纥"},
    {"紈", "纨"},
    {"紉", "纫"},
    {"紋", "纹"},
    {"納", "纳"},
    {"紐", "纽"},
    {"紓", "纾"},
    {"純", "纯"},
    {"紕", "纰"},
    {"紗", "纱"},
    {"紙", "纸"},
    {"級", "级"},
    {"紛", "纷"},
    {"紜", "纭"},
    {"紡", "纺"},
    {"紮", "扎"},
    {"細", "细"},
    {"紱", "绂"},
    {"紲", "绁"},
    {"紳", "绅"},
    {"紹", "绍"},
    {"紺", "绀"},
    {"紼", "绋"},
    {"紿", "绐"},
    {"絀", "绌"},
    {"終", "终"},
    {"絃", "弦"},
    {"組", "组"},
    {"絆", "绊"},
    {"絎", "绗"},
    {"結", "结"},
    {"絕", "绝"},
    {"絛", "绦"},
    {"絞", "绞"},
    {"絡", "络"},
    {"絢", "绚"},
    {"給", "给"},
    {"絨", "绒"},
    {"統", "统"},
    {"絲", "丝"},
    {"絳", "绛"},
    {"絹", "绢"},
    {"綁", "绑"},
    {"綃", "绡"},
    {"綆", "绠"},
    {"綈", "绨"},
    {"綏", "绥"},
    {"綑", "捆"},
    {"經", "经"},
    {"綜", "综"},
    {"綞", "缍"},
    {"綠", "绿"},
    {"綢", "绸"},
    {"綣", "绻"},
    {"綬", "绶"},
    {"維", "维"},
    {"綰", "绾"},
    {"綱", "纲"},
    {"網", "网"},
    {"綴", "缀"},
    {"綵", "彩"},
    {"綸", "纶"},
    {"綹", "绺"},
    {"綺", "绮"},
    {"綻", "绽"},
    {"綽", "绰"},
    {"綾", "绫"},
    {"綿", "绵"},
    {"緄", "绲"},
    {"緇", "缁"},
    {"緊", "紧"},
    {"緋", "绯"},
    {"緒", "绪"},
    {"緗", "缃"},
    {"緘", "缄"},
    {"緙", "缂"},
    {"線", "线"},
    {"緝", "缉"},
    {"緞", "缎"},
    {"締", "缔"},
    {"緡", "缗"},
    {"緣", "缘"},
    {"緦", "缌"},
    {"編", "编"},
    {"緩", "缓"},
    {"緬", "缅"},
    {"緯", "纬"},
    {"緱", "缑"},
    {"緲", "缈"},
    {"練", "练"},
    {"緶", "缏"},
    {"緹", "缇"},
    {"緻", "致"},
    {"縈", "萦"},
    {"縉", "缙"},
    {"縊", "缢"},
    {"縋", "缒"},
    {"縐", "绉"},
    {"縑", "缣"},
    {"縚", "绦"},
    {"縛", "缚"},
    {"縝", "缜"},
    {"縞", "缟"},
    {"縟", "缛"},
    {"縣", "县"},
    {"縫", "缝"},
    {"縭", "缡"},
    {"縮", "缩"},
    {"縯", "演"},
    {"縱", "纵"},
    {"縲", "缧"},
    {"縳", "缚"},
    {"縴", "纤"},
    {"縵", "缦"},
    {"縶", "絷"},
    {"縷", "缕"},
    {"縹", "缥"},
    {"總", "总"},
    {"績", "绩"},
    {"繃", "绷"},
    {"繅", "缫"},
    {"繆", "缪"},
    {"繈", "襁"},
    {"繒", "缯"},
    {"織", "织"},
    {"繕", "缮"},
    {"繙", "翻"},
    {"繚", "缭"},
    {"繞", "绕"},
    {"繡", "绣"},
    {"繢", "缋"},
    {"繩", "绳"},
    {"繪", "绘"},
    {"繫", "系"},
    {"繭", "茧"},
    {"繯", "缳"},
    {"繰", "缲"},
    {"繳", "缴"},
    {"繹", "绎"},
    {"繼", "继"},
    {"繽", "缤"},
    {"繾", "缱"},
    {"纈", "缬"},
    {"纊", "纩"},
    {"續", "续"},
    {"纍", "累"},
    {"纏", "缠"},
    {"纓", "缨"},
    {"纔", "才"},
    {"纖", "纤"},
    {"纘", "缵"},
    {"纜", "缆"},
    {"缽", "钵"},
    {"缾", "瓶"},
    {"罈", "坛"},
    {"罌", "罂"},
    {"罦", "罘"},
    {"罰", "罚"},
    {"罵", "骂"},
    {"罷", "罢"},
    {"羅", "罗"},
    {"羆", "罴"},
    {"羈", "羁"},
    {"羋", "芈"},
    {"羥", "羟"},
    {"羨", "羡"},
    {"義", "义"},
    {"羶", "膻"},
    {"習", "习"},
    {"翹", "翘"},
    {"耑", "端"},
    {"耡", "助"},
    {"耤", "藉"},
    {"耬", "耧"},
    {"聖", "圣"},
    {"聞", "闻"},
    {"聯", "联"},
    {"聰", "聪"},
    {"聲", "声"},
    {"聳", "耸"},
    {"聵", "聩"},
    {"聶", "聂"},
    {"職", "职"},
    {"聹", "聍"},
    {"聽", "听"},
    {"聾", "聋"},
    {"肅", "肃"},
    {"肏", "操"},
    {"肐", "胳"},
    {"胇", "肺"},
    {"胊", "朐"},
    {"脅", "胁"},
    {"脈", "脉"},
    {"脛", "胫"},
    {"脣", "唇"},
    {"脩", "修"},
    {"脫", "脱"},
    {"脹", "胀"},
    {"腎", "肾"},
    {"腡", "脶"},
    {"腦", "脑"},
    {"腫", "肿"},
    {"腳", "脚"},
    {"腸", "肠"},
    {"膃", "腽"},
    {"膆", "嗉"},
    {"膚", "肤"},
    {"膠", "胶"},
    {"膩", "腻"},
    {"膽", "胆"},
    {"膾", "脍"},
    {"膿", "脓"},
    {"臉", "脸"},
    {"臍", "脐"},
    {"臏", "膑"},
    {"臕", "膘"},
    {"臘", "腊"},
    {"臙", "胭"},
    {"臚", "胪"},
    {"臟", "脏"},
    {"臠", "脔"},
    {"臥", "卧"},
    {"臨", "临"},
    {"臺", "台"},
    {"與", "与"},
    {"興", "兴"},
    {"舉", "举"},
    {"舊", "旧"},
    {"舋", "衅"},
    {"舖", "铺"},
    {"艙", "舱"},
    {"艣", "橹"},
    {"艤", "舣"},
    {"艦", "舰"},
    {"艫", "舻"},
    {"艱", "艰"},
    {"艷", "艳"},
    {"艸", "艹"},
    {"芻", "刍"},
    {"苧", "苎"},
    {"苺", "莓"},
    {"茍", "苟"},
    {"茲", "兹"},
    {"荅", "答"},
    {"荊", "荆"},
    {"荳", "豆"},
    {"莊", "庄"},
    {"莖", "茎"},
    {"莢", "荚"},
    {"莧", "苋"},
    {"菫", "堇"},
    {"華", "华"},
    {"菴", "庵"},
    {"萇", "苌"},
    {"萊", "莱"},
    {"萬", "万"},
    {"萵", "莴"},
    {"葉", "叶"},
    {"葒", "荭"},
    {"葦", "苇"},
    {"葯", "药"},
    {"葷", "荤"},
    {"蒐", "搜"},
    {"蒔", "莳"},
    {"蒞", "莅"},
    {"蒼", "苍"},
    {"蓀", "荪"},
    {"蓆", "席"},
    {"蓋", "盖"},
    {"蓮", "莲"},
    {"蓯", "苁"},
    {"蓴", "莼"},
    {"蓽", "荜"},
    {"蔆", "菱"},
    {"蔔", "卜"},
    {"蔞", "蒌"},
    {"蔣", "蒋"},
    {"蔥", "葱"},
    {"蔦", "茑"},
    {"蔭", "荫"},
    {"蕁", "荨"},
    {"蕆", "蒇"},
    {"蕎", "荞"},
    {"蕓", "芸"},
    {"蕕", "莸"},
    {"蕘", "荛"},
    {"蕢", "蒉"},
    {"蕩", "荡"},
    {"蕪", "芜"},
    {"蕭", "萧"},
    {"蕷", "蓣"},
    {"薈", "荟"},
    {"薊", "蓟"},
    {"薌", "芗"},
    {"薑", "姜"},
    {"薔", "蔷"},
    {"薙", "剃"},
    {"薟", "莶"},
    {"薦", "荐"},
    {"薩", "萨"},
    {"薺", "荠"},
    {"藍", "蓝"},
    {"藎", "荩"},
    {"藚", "荬"},
    {"藝", "艺"},
    {"藥", "药"},
    {"藪", "薮"},
    {"藶", "苈"},
    {"藷", "薯"},
    {"藹", "蔼"},
    {"藺", "蔺"},
    {"蘄", "蕲"},
    {"蘆", "芦"},
    {"蘇", "苏"},
    {"蘊", "蕴"},
    {"蘋", "苹"},
    {"蘗", "蘖"},
    {"蘚", "藓"},
    {"蘞", "蔹"},
    {"蘢", "茏"},
    {"蘭", "兰"},
    {"蘺", "蓠"},
    {"蘿", "萝"},
    {"處", "处"},
    {"虖", "呼"},
    {"虛", "虚"},
    {"虜", "虏"},
    {"號", "号"},
    {"虧", "亏"},
    {"虯", "虬"},
    {"蛺", "蛱"},
    {"蛻", "蜕"},
    {"蜆", "蚬"},
    {"蜺", "霓"},
    {"蝕", "蚀"},
    {"蝟", "猬"},
    {"蝦", "虾"},
    {"蝨", "虱"},
    {"蝸", "蜗"},
    {"螄", "蛳"},
    {"螞", "蚂"},
    {"螢", "萤"},
    {"螻", "蝼"},
    {"蟄", "蛰"},
    {"蟈", "蝈"},
    {"蟣", "虮"},
    {"蟬", "蝉"},
    {"蟯", "蛲"},
    {"蟲", "虫"},
    {"蟶", "蛏"},
    {"蟺", "蟮"},
    {"蟻", "蚁"},
    {"蠅", "蝇"},
    {"蠆", "虿"},
    {"蠍", "蝎"},
    {"蠐", "蛴"},
    {"蠑", "蝾"},
    {"蠔", "蚝"},
    {"蠟", "蜡"},
    {"蠣", "蛎"},
    {"蠱", "蛊"},
    {"蠶", "蚕"},
    {"蠷", "蠼"},
    {"蠻", "蛮"},
    {"衊", "蔑"},
    {"衒", "炫"},
    {"術", "术"},
    {"衚", "胡"},
    {"衛", "卫"},
    {"衝", "冲"},
    {"衹", "只"},
    {"袞", "衮"},
    {"袪", "祛"},
    {"裊", "袅"},
    {"裏", "里"},
    {"補", "补"},
    {"裝", "装"},
    {"裡", "里"},
    {"製", "制"},
    {"複", "复"},
    {"褎", "袖"},
    {"褲", "裤"},
    {"褳", "裢"},
    {"褸", "褛"},
    {"褻", "亵"},
    {"襉", "裥"},
    {"襖", "袄"},
    {"襝", "裣"},
    {"襠", "裆"},
    {"襤", "褴"},
    {"襪", "袜"},
    {"襬", "摆"},
    {"襯", "衬"},
    {"襲", "袭"},
    {"襾", "西"},
    {"覈", "核"},
    {"見", "见"},
    {"規", "规"},
    {"覓", "觅"},
    {"視", "视"},
    {"覘", "觇"},
    {"覜", "眺"},
    {"覡", "觋"},
    {"覦", "觎"},
    {"親", "亲"},
    {"覬", "觊"},
    {"覯", "觏"},
    {"覲", "觐"},
    {"覷", "觑"},
    {"覺", "觉"},
    {"覽", "览"},
    {"覿", "觌"},
    {"觀", "观"},
    {"觔", "筋"},
    {"觝", "抵"},
    {"觴", "觞"},
    {"觶", "觯"},
    {"觸", "触"},
    {"訂", "订"},
    {"訃", "讣"},
    {"計", "计"},
    {"訊", "讯"},
    {"訌", "讧"},
    {"討", "讨"},
    {"訐", "讦"},
    {"訓", "训"},
    {"訕", "讪"},
    {"訖", "讫"},
    {"託", "托"},
    {"記", "记"},
    {"訛", "讹"},
    {"訝", "讶"},
    {"訟", "讼"},
    {"訢", "欣"},
    {"訣", "诀"},
    {"訥", "讷"},
    {"訪", "访"},
    {"設", "设"},
    {"許", "许"},
    {"訴", "诉"},
    {"訶", "诃"},
    {"診", "诊"},
    {"註", "注"},
    {"証", "证"},
    {"詁", "诂"},
    {"詆", "诋"},
    {"詎", "讵"},
    {"詐", "诈"},
    {"詒", "诒"},
    {"詔", "诏"},
    {"評", "评"},
    {"詘", "诎"},
    {"詛", "诅"},
    {"詞", "词"},
    {"詠", "咏"},
    {"詡", "诩"},
    {"詢", "询"},
    {"詣", "诣"},
    {"試", "试"},
    {"詩", "诗"},
    {"詫", "诧"},
    {"詬", "诟"},
    {"詭", "诡"},
    {"詮", "诠"},
    {"詰", "诘"},
    {"話", "话"},
    {"該", "该"},
    {"詳", "详"},
    {"詵", "诜"},
    {"詶", "酬"},
    {"詻", "咯"},
    {"詼", "诙"},
    {"詿", "诖"},
    {"誄", "诔"},
    {"誅", "诛"},
    {"誆", "诓"},
    {"誇", "夸"},
    {"誌", "志"},
    {"認", "认"},
    {"誑", "诳"},
    {"誒", "诶"},
    {"誕", "诞"},
    {"誘", "诱"},
    {"誚", "诮"},
    {"語", "语"},
    {"誠", "诚"},
    {"誡", "诫"},
    {"誣", "诬"},
    {"誤", "误"},
    {"誥", "诰"},
    {"誦", "诵"},
    {"誨", "诲"},
    {"說", "说"},
    {"誰", "谁"},
    {"課", "课"},
    {"誶", "谇"},
    {"誹", "诽"},
    {"誼", "谊"},
    {"調", "调"},
    {"諂", "谄"},
    {"諄", "谆"},
    {"談", "谈"},
    {"諉", "诿"},
    {"請", "请"},
    {"諍", "诤"},
    {"諏", "诹"},
    {"諑", "诼"},
    {"諒", "谅"},
    {"論", "论"},
    {"諗", "谂"},
    {"諛", "谀"},
    {"諜", "谍"},
    {"諞", "谝"},
    {"諠", "喧"},
    {"諢", "诨"},
    {"諤", "谔"},
    {"諦", "谛"},
    {"諧", "谐"},
    {"諫", "谏"},
    {"諭", "谕"},
    {"諮", "谘"},
    {"諱", "讳"},
    {"諳", "谙"},
    {"諶", "谌"},
    {"諷", "讽"},
    {"諸", "诸"},
    {"諺", "谚"},
    {"諼", "谖"},
    {"諾", "诺"},
    {"謀", "谋"},
    {"謁", "谒"},
    {"謂", "谓"},
    {"謄", "誊"},
    {"謅", "诌"},
    {"謊", "谎"},
    {"謎", "谜"},
    {"謐", "谧"},
    {"謔", "谑"},
    {"謖", "谡"},
    {"謗", "谤"},
    {"謙", "谦"},
    {"謚", "谥"},
    {"講", "讲"},
    {"謝", "谢"},
    {"謠", "谣"},
    {"謨", "谟"},
    {"謫", "谪"},
    {"謬", "谬"},
    {"謳", "讴"},
    {"謹", "谨"},
    {"謼", "呼"},
    {"謾", "谩"},
    {"譁", "哗"},
    {"譆", "嘻"},
    {"證", "证"},
    {"譎", "谲"},
    {"譏", "讥"},
    {"譔", "撰"},
    {"譖", "谮"},
    {"識", "识"},
    {"譙", "谯"},
    {"譚", "谭"},
    {"譜", "谱"},
    {"譟", "噪"},
    {"譫", "谵"},
    {"譭", "毁"},
    {"譯", "译"},
    {"議", "议"},
    {"譴", "谴"},
    {"護", "护"},
    {"譽", "誉"},
    {"譾", "谫"},
    {"讀", "读"},
    {"變", "变"},
    {"讌", "宴"},
    {"讎", "雠"},
    {"讒", "谗"},
    {"讓", "让"},
    {"讕", "谰"},
    {"讖", "谶"},
    {"讚", "赞"},
    {"讜", "谠"},
    {"讞", "谳"},
    {"谿", "溪"},
    {"豈", "岂"},
    {"豎", "竖"},
    {"豐", "丰"},
    {"豔", "艳"},
    {"豖", "亍"},
    {"豬", "猪"},
    {"貍", "狸"},
    {"貓", "猫"},
    {"貝", "贝"},
    {"貞", "贞"},
    {"負", "负"},
    {"財", "财"},
    {"貢", "贡"},
    {"貧", "贫"},
    {"貨", "货"},
    {"販", "贩"},
    {"貪", "贪"},
    {"貫", "贯"},
    {"責", "责"},
    {"貯", "贮"},
    {"貰", "贳"},
    {"貲", "赀"},
    {"貳", "贰"},
    {"貴", "贵"},
    {"貶", "贬"},
    {"買", "买"},
    {"貸", "贷"},
    {"貺", "贶"},
    {"費", "费"},
    {"貼", "贴"},
    {"貽", "贻"},
    {"貿", "贸"},
    {"賀", "贺"},
    {"賁", "贲"},
    {"賂", "赂"},
    {"賃", "赁"},
    {"賄", "贿"},
    {"賅", "赅"},
    {"資", "资"},
    {"賈", "贾"},
    {"賊", "贼"},
    {"賑", "赈"},
    {"賒", "赊"},
    {"賓", "宾"},
    {"賕", "赇"},
    {"賙", "周"},
    {"賚", "赉"},
    {"賜", "赐"},
    {"賞", "赏"},
    {"賠", "赔"},
    {"賡", "赓"},
    {"賢", "贤"},
    {"賣", "卖"},
    {"賤", "贱"},
    {"賦", "赋"},
    {"賧", "赕"},
    {"質", "质"},
    {"賬", "账"},
    {"賭", "赌"},
    {"賴", "赖"},
    {"賸", "剩"},
    {"賺", "赚"},
    {"賻", "赙"},
    {"購", "购"},
    {"賽", "赛"},
    {"賾", "赜"},
    {"贄", "贽"},
    {"贅", "赘"},
    {"贈", "赠"},
    {"贊", "赞"},
    {"贍", "赡"},
    {"贏", "赢"},
    {"贐", "赆"},
    {"贓", "赃"},
    {"贖", "赎"},
    {"贗", "赝"},
    {"贛", "赣"},
    {"趕", "赶"},
    {"趙", "赵"},
    {"趨", "趋"},
    {"趲", "趱"},
    {"跡", "迹"},
    {"跼", "局"},
    {"踐", "践"},
    {"踡", "蜷"},
    {"踫", "碰"},
    {"踰", "逾"},
    {"踴", "踊"},
    {"蹌", "跄"},
    {"蹕", "跸"},
    {"蹟", "迹"},
    {"蹠", "跖"},
    {"蹣", "蹒"},
    {"蹤", "踪"},
    {"蹧", "糟"},
    {"蹺", "跷"},
    {"躉", "趸"},
    {"躊", "踌"},
    {"躋", "跻"},
    {"躍", "跃"},
    {"躑", "踯"},
    {"躒", "跞"},
    {"躓", "踬"},
    {"躕", "蹰"},
    {"躚", "跹"},
    {"躡", "蹑"},
    {"躥", "蹿"},
    {"躦", "躜"},
    {"躪", "躏"},
    {"軀", "躯"},
    {"車", "车"},
    {"軋", "轧"},
    {"軌", "轨"},
    {"軍", "军"},
    {"軒", "轩"},
    {"軔", "轫"},
    {"軛", "轭"},
    {"軟", "软"},
    {"軫", "轸"},
    {"軸", "轴"},
    {"軹", "轵"},
    {"軺", "轺"},
    {"軻", "轲"},
    {"軼", "轶"},
    {"軾", "轼"},
    {"較", "较"},
    {"輅", "辂"},
    {"輇", "辁"},
    {"載", "载"},
    {"輊", "轾"},
    {"輒", "辄"},
    {"輓", "挽"},
    {"輔", "辅"},
    {"輕", "轻"},
    {"輛", "辆"},
    {"輜", "辎"},
    {"輝", "辉"},
    {"輞", "辋"},
    {"輟", "辍"},
    {"輥", "辊"},
    {"輦", "辇"},
    {"輩", "辈"},
    {"輪", "轮"},
    {"輯", "辑"},
    {"輳", "辏"},
    {"輸", "输"},
    {"輻", "辐"},
    {"輾", "辗"},
    {"輿", "舆"},
    {"轂", "毂"},
    {"轄", "辖"},
    {"轅", "辕"},
    {"轆", "辘"},
    {"轉", "转"},
    {"轍", "辙"},
    {"轎", "轿"},
    {"轔", "辚"},
    {"轟", "轰"},
    {"轡", "辔"},
    {"轢", "轹"},
    {"轤", "轳"},
    {"辦", "办"},
    {"辭", "辞"},
    {"辮", "辫"},
    {"辯", "辩"},
    {"農", "农"},
    {"迆", "迤"},
    {"迴", "回"},
    {"迺", "乃"},
    {"逕", "迳"},
    {"這", "这"},
    {"連", "连"},
    {"週", "周"},
    {"進", "进"},
    {"遊", "游"},
    {"運", "运"},
    {"過", "过"},
    {"達", "达"},
    {"違", "违"},
    {"遙", "遥"},
    {"遜", "逊"},
    {"遞", "递"},
    {"遠", "远"},
    {"適", "适"},
    {"遲", "迟"},
    {"遷", "迁"},
    {"選", "选"},
    {"遺", "遗"},
    {"遼", "辽"},
    {"邁", "迈"},
    {"還", "还"},
    {"邇", "迩"},
    {"邊", "边"},
    {"邏", "逻"},
    {"邐", "逦"},
    {"郟", "郏"},
    {"郵", "邮"},
    {"鄆", "郓"},
    {"鄉", "乡"},
    {"鄒", "邹"},
    {"鄔", "邬"},
    {"鄖", "郧"},
    {"鄧", "邓"},
    {"鄭", "郑"},
    {"鄰", "邻"},
    {"鄲", "郸"},
    {"鄴", "邺"},
    {"鄶", "郐"},
    {"鄺", "邝"},
    {"酈", "郦"},
    {"酖", "鸩"},
    {"醃", "腌"},
    {"醆", "盏"},
    {"醜", "丑"},
    {"醞", "酝"},
    {"醫", "医"},
    {"醬", "酱"},
    {"醱", "发"},
    {"醼", "宴"},
    {"釀", "酿"},
    {"釁", "衅"},
    {"釃", "酾"},
    {"釅", "酽"},
    {"釆", "采"},
    {"釋", "释"},
    {"釐", "厘"},
    {"釓", "钆"},
    {"釔", "钇"},
    {"釕", "钌"},
    {"釗", "钊"},
    {"釘", "钉"},
    {"釙", "钋"},
    {"針", "针"},
    {"釣", "钓"},
    {"釤", "钐"},
    {"釦", "扣"},
    {"釧", "钏"},
    {"釩", "钒"},
    {"釬", "钎"},
    {"釵", "钗"},
    {"釷", "钍"},
    {"釹", "钕"},
    {"鈀", "钯"},
    {"鈁", "钫"},
    {"鈄", "钭"},
    {"鈉", "钠"},
    {"鈍", "钝"},
    {"鈐", "钤"},
    {"鈑", "钣"},
    {"鈔", "钞"},
    {"鈕", "钮"},
    {"鈞", "钧"},
    {"鈣", "钙"},
    {"鈥", "钬"},
    {"鈦", "钛"},
    {"鈧", "钪"},
    {"鈮", "铌"},
    {"鈰", "铈"},
    {"鈳", "钶"},
    {"鈴", "铃"},
    {"鈷", "钴"},
    {"鈸", "钹"},
    {"鈹", "铍"},
    {"鈺", "钰"},
    {"鈽", "钸"},
    {"鈾", "铀"},
    {"鈿", "钿"},
    {"鉀", "钾"},
    {"鉅", "巨"},
    {"鉆", "钻"},
    {"鉈", "铊"},
    {"鉉", "铉"},
    {"鉋", "刨"},
    {"鉍", "铋"},
    {"鉑", "铂"},
    {"鉗", "钳"},
    {"鉚", "铆"},
    {"鉛", "铅"},
    {"鉞", "钺"},
    {"鉤", "钩"},
    {"鉦", "钲"},
    {"鉬", "钼"},
    {"鉭", "钽"},
    {"鉸", "铰"},
    {"鉺", "铒"},
    {"鉻", "铬"},
    {"鉿", "铪"},
    {"銀", "银"},
    {"銃", "铳"},
    {"銅", "铜"},
    {"銑", "铣"},
    {"銓", "铨"},
    {"銖", "铢"},
    {"銘", "铭"},
    {"銚", "铫"},
    {"銜", "衔"},
    {"銠", "铑"},
    {"銣", "铷"},
    {"銥", "铱"},
    {"銦", "铟"},
    {"銨", "铵"},
    {"銩", "铥"},
    {"銪", "铕"},
    {"銫", "铯"},
    {"銬", "铐"},
    {"銲", "焊"},
    {"銳", "锐"},
    {"銷", "销"},
    {"銹", "锈"},
    {"銻", "锑"},
    {"銼", "锉"},
    {"鋁", "铝"},
    {"鋃", "锒"},
    {"鋅", "锌"},
    {"鋇", "钡"},
    {"鋌", "铤"},
    {"鋏", "铗"},
    {"鋒", "锋"},
    {"鋝", "锊"},
    {"鋟", "锓"},
    {"鋤", "锄"},
    {"鋦", "锔"},
    {"鋨", "锇"},
    {"鋪", "铺"},
    {"鋮", "铖"},
    {"鋯", "锆"},
    {"鋰", "锂"},
    {"鋱", "铽"},
    {"鋸", "锯"},
    {"鋻", "鉴"},
    {"鋼", "钢"},
    {"錁", "锞"},
    {"錄", "录"},
    {"錆", "锖"},
    {"錈", "锩"},
    {"錐", "锥"},
    {"錒", "锕"},
    {"錕", "锟"},
    {"錘", "锤"},
    {"錙", "锱"},
    {"錚", "铮"},
    {"錛", "锛"},
    {"錟", "锬"},
    {"錠", "锭"},
    {"錢", "钱"},
    {"錦", "锦"},
    {"錨", "锚"},
    {"錫", "锡"},
    {"錮", "锢"},
    {"錯", "错"},
    {"錳", "锰"},
    {"錶", "表"},
    {"錸", "铼"},
    {"鍆", "钔"},
    {"鍇", "锴"},
    {"鍊", "炼"},
    {"鍋", "锅"},
    {"鍍", "镀"},
    {"鍔", "锷"},
    {"鍘", "铡"},
    {"鍛", "锻"},
    {"鍤", "锸"},
    {"鍥", "锲"},
    {"鍬", "锹"},
    {"鍰", "锾"},
    {"鍵", "键"},
    {"鍶", "锶"},
    {"鍺", "锗"},
    {"鍼", "针"},
    {"鍾", "钟"},
    {"鎂", "镁"},
    {"鎊", "镑"},
    {"鎌", "镰"},
    {"鎔", "熔"},
    {"鎖", "锁"},
    {"鎗", "枪"},
    {"鎘", "镉"},
    {"鎚", "锤"},
    {"鎢", "钨"},
    {"鎣", "蓥"},
    {"鎦", "镏"},
    {"鎧", "铠"},
    {"鎩", "铩"},
    {"鎪", "锼"},
    {"鎬", "镐"},
    {"鎮", "镇"},
    {"鎰", "镒"},
    {"鎳", "镍"},
    {"鎵", "镓"},
    {"鏃", "镞"},
    {"鏇", "镟"},
    {"鏈", "链"},
    {"鏌", "镆"},
    {"鏍", "镙"},
    {"鏑", "镝"},
    {"鏗", "铿"},
    {"鏘", "锵"},
    {"鏜", "镗"},
    {"鏝", "镘"},
    {"鏞", "镛"},
    {"鏟", "铲"},
    {"鏡", "镜"},
    {"鏢", "镖"},
    {"鏤", "镂"},
    {"鏨", "錾"},
    {"鏵", "铧"},
    {"鏷", "镤"},
    {"鏹", "镪"},
    {"鏽", "锈"},
    {"鐃", "铙"},
    {"鐉", "铣"},
    {"鐋", "铴"},
    {"鐐", "镣"},
    {"鐒", "铹"},
    {"鐓", "镦"},
    {"鐔", "镡"},
    {"鐘", "钟"},
    {"鐙", "镫"},
    {"鐠", "镨"},
    {"鐨", "镄"},
    {"鐫", "镌"},
    {"鐮", "镰"},
    {"鐲", "镯"},
    {"鐳", "镭"},
    {"鐵", "铁"},
    {"鐶", "环"},
    {"鐸", "铎"},
    {"鐺", "铛"},
    {"鐿", "镱"},
    {"鑄", "铸"},
    {"鑊", "镬"},
    {"鑌", "镔"},
    {"鑑", "鉴"},
    {"鑒", "鉴"},
    {"鑠", "铄"},
    {"鑣", "镳"},
    {"鑤", "刨"},
    {"鑪", "炉"},
    {"鑭", "镧"},
    {"鑰", "钥"},
    {"鑲", "镶"},
    {"鑵", "罐"},
    {"鑷", "镊"},
    {"鑼", "锣"},
    {"鑽", "钻"},
    {"鑾", "銮"},
    {"鑿", "凿"},
    {"長", "长"},
    {"門", "门"},
    {"閂", "闩"},
    {"閃", "闪"},
    {"閆", "闫"},
    {"閉", "闭"},
    {"開", "开"},
    {"閌", "闶"},
    {"閎", "闳"},
    {"閏", "闰"},
    {"閑", "闲"},
    {"閒", "闲"},
    {"間", "间"},
    {"閔", "闵"},
    {"閘", "闸"},
    {"閡", "阂"},
    {"閣", "阁"},
    {"閤", "合"},
    {"閥", "阀"},
    {"閨", "闺"},
    {"閩", "闽"},
    {"閫", "阃"},
    {"閬", "阆"},
    {"閭", "闾"},
    {"閱", "阅"},
    {"閶", "阊"},
    {"閹", "阉"},
    {"閻", "阎"},
    {"閼", "阏"},
    {"閽", "阍"},
    {"閾", "阈"},
    {"閿", "阌"},
    {"闃", "阒"},
    {"闆", "板"},
    {"闇", "暗"},
    {"闈", "闱"},
    {"闊", "阔"},
    {"闋", "阕"},
    {"闌", "阑"},
    {"闐", "阗"},
    {"闔", "阖"},
    {"闕", "阙"},
    {"闖", "闯"},
    {"關", "关"},
    {"闞", "阚"},
    {"闡", "阐"},
    {"闢", "辟"},
    {"闥", "闼"},
    {"阨", "厄"},
    {"阬", "坑"},
    {"阯", "址"},
    {"陏", "隋"},
    {"陘", "陉"},
    {"陝", "陕"},
    {"陞", "升"},
    {"陣", "阵"},
    {"陰", "阴"},
    {"陳", "陈"},
    {"陸", "陆"},
    {"陽", "阳"},
    {"隄", "堤"},
    {"隉", "陧"},
    {"隊", "队"},
    {"階", "阶"},
    {"隕", "陨"},
    {"際", "际"},
    {"隤", "颓"},
    {"隨", "随"},
    {"險", "险"},
    {"隱", "隐"},
    {"隴", "陇"},
    {"隸", "隶"},
    {"隻", "只"},
    {"雋", "隽"},
    {"雖", "虽"},
    {"雙", "双"},
    {"雛", "雏"},
    {"雜", "杂"},
    {"雞", "鸡"},
    {"離", "离"},
    {"難", "难"},
    {"雲", "云"},
    {"電", "电"},
    {"霑", "沾"},
    {"霤", "溜"},
    {"霧", "雾"},
    {"霽", "霁"},
    {"靂", "雳"},
    {"靄", "霭"},
    {"靈", "灵"},
    {"靚", "靓"},
    {"靜", "静"},
    {"靦", "腼"},
    {"靨", "靥"},
    {"鞏", "巩"},
    {"鞦", "秋"},
    {"韁", "缰"},
    {"韃", "鞑"},
    {"韆", "千"},
    {"韉", "鞯"},
    {"韋", "韦"},
    {"韌", "韧"},
    {"韓", "韩"},
    {"韙", "韪"},
    {"韜", "韬"},
    {"韞", "韫"},
    {"韻", "韵"},
    {"響", "响"},
    {"頁", "页"},
    {"頂", "顶"},
    {"頃", "顷"},
    {"項", "项"},
    {"順", "顺"},
    {"頇", "顸"},
    {"須", "须"},
    {"頊", "顼"},
    {"頌", "颂"},
    {"頎", "颀"},
    {"頏", "颃"},
    {"預", "预"},
    {"頑", "顽"},
    {"頒", "颁"},
    {"頓", "顿"},
    {"頗", "颇"},
    {"領", "领"},
    {"頜", "颌"},
    {"頡", "颉"},
    {"頤", "颐"},
    {"頦", "颏"},
    {"頫", "俯"},
    {"頭", "头"},
    {"頰", "颊"},
    {"頷", "颔"},
    {"頸", "颈"},
    {"頹", "颓"},
    {"頻", "频"},
    {"顆", "颗"},
    {"題", "题"},
    {"額", "额"},
    {"顎", "颚"},
    {"顏", "颜"},
    {"顓", "颛"},
    {"願", "愿"},
    {"顙", "颡"},
    {"顛", "颠"},
    {"類", "类"},
    {"顢", "颟"},
    {"顥", "颢"},
    {"顧", "顾"},
    {"顫", "颤"},
    {"顯", "显"},
    {"顰", "颦"},
    {"顱", "颅"},
    {"顳", "颞"},
    {"顴", "颧"},
    {"風", "风"},
    {"颮", "飑"},
    {"颯", "飒"},
    {"颱", "台"},
    {"颳", "刮"},
    {"颶", "飓"},
    {"颺", "扬"},
    {"颼", "飕"},
    {"飄", "飘"},
    {"飆", "飙"},
    {"飛", "飞"},
    {"飢", "饥"},
    {"飩", "饨"},
    {"飪", "饪"},
    {"飫", "饫"},
    {"飭", "饬"},
    {"飯", "饭"},
    {"飲", "饮"},
    {"飴", "饴"},
    {"飼", "饲"},
    {"飽", "饱"},
    {"飾", "饰"},
    {"餃", "饺"},
    {"餅", "饼"},
    {"餈", "糍"},
    {"餉", "饷"},
    {"養", "养"},
    {"餌", "饵"},
    {"餑", "饽"},
    {"餒", "馁"},
    {"餓", "饿"},
    {"餔", "哺"},
    {"餘", "余"},
    {"餚", "肴"},
    {"餛", "馄"},
    {"餞", "饯"},
    {"餡", "馅"},
    {"館", "馆"},
    {"餬", "糊"},
    {"餱", "糇"},
    {"餳", "饧"},
    {"餵", "喂"},
    {"餼", "饩"},
    {"餽", "馈"},
    {"餾", "馏"},
    {"餿", "馊"},
    {"饃", "馍"},
    {"饅", "馒"},
    {"饈", "馐"},
    {"饉", "馑"},
    {"饋", "馈"},
    {"饌", "馔"},
    {"饑", "饥"},
    {"饒", "饶"},
    {"饗", "飨"},
    {"饜", "餍"},
    {"饞", "馋"},
    {"饟", "饷"},
    {"馬", "马"},
    {"馭", "驭"},
    {"馮", "冯"},
    {"馱", "驮"},
    {"馳", "驰"},
    {"馴", "驯"},
    {"駁", "驳"},
    {"駐", "驻"},
    {"駑", "驽"},
    {"駒", "驹"},
    {"駔", "驵"},
    {"駕", "驾"},
    {"駘", "骀"},
    {"駙", "驸"},
    {"駛", "驶"},
    {"駝", "驼"},
    {"駟", "驷"},
    {"駢", "骈"},
    {"駭", "骇"},
    {"駮", "驳"},
    {"駱", "骆"},
    {"駿", "骏"},
    {"騁", "骋"},
    {"騃", "呆"},
    {"騅", "骓"},
    {"騍", "骒"},
    {"騎", "骑"},
    {"騏", "骐"},
    {"騖", "骛"},
    {"騙", "骗"},
    {"騣", "鬃"},
    {"騫", "骞"},
    {"騭", "骘"},
    {"騮", "骝"},
    {"騰", "腾"},
    {"騶", "驺"},
    {"騷", "骚"},
    {"騸", "骟"},
    {"騾", "骡"},
    {"驀", "蓦"},
    {"驁", "骜"},
    {"驂", "骖"},
    {"驃", "骠"},
    {"驄", "骢"},
    {"驅", "驱"},
    {"驊", "骅"},
    {"驍", "骁"},
    {"驏", "骣"},
    {"驕", "骄"},
    {"驗", "验"},
    {"驚", "惊"},
    {"驛", "驿"},
    {"驟", "骤"},
    {"驢", "驴"},
    {"驤", "骧"},
    {"驥", "骥"},
    {"驪", "骊"},
    {"骯", "肮"},
    {"髏", "髅"},
    {"髒", "脏"},
    {"體", "体"},
    {"髕", "髌"},
    {"髖", "髋"},
    {"髣", "仿"},
    {"髮", "发"},
    {"鬆", "松"},
    {"鬍", "胡"},
    {"鬚", "须"},
    {"鬢", "鬓"},
    {"鬥", "斗"},
    {"鬧", "闹"},
    {"鬨", "哄"},
    {"鬩", "阋"},
    {"鬮", "阄"},
    {"鬱", "郁"},
    {"魎", "魉"},
    {"魘", "魇"},
    {"魚", "鱼"},
    {"魨", "豚"},
    {"魯", "鲁"},
    {"魴", "鲂"},
    {"魷", "鱿"},
    {"鮐", "鲐"},
    {"鮑", "鲍"},
    {"鮒", "鲋"},
    {"鮚", "鲒"},
    {"鮞", "鲕"},
    {"鮪", "鲔"},
    {"鮫", "鲛"},
    {"鮭", "鲑"},
    {"鮮", "鲜"},
    {"鯀", "鲧"},
    {"鯁", "鲠"},
    {"鯇", "鲩"},
    {"鯉", "鲤"},
    {"鯊", "鲨"},
    {"鯔", "鲻"},
    {"鯖", "鲭"},
    {"鯗", "鲞"},
    {"鯛", "鲷"},
    {"鯡", "鲱"},
    {"鯢", "鲵"},
    {"鯤", "鲲"},
    {"鯧", "鲳"},
    {"鯨", "鲸"},
    {"鯪", "鲮"},
    {"鯫", "鲰"},
    {"鯰", "鲇"},
    {"鯽", "鲫"},
    {"鰈", "鲽"},
    {"鰉", "鳇"},
    {"鰍", "鳅"},
    {"鰒", "鳆"},
    {"鰓", "鳃"},
    {"鰣", "鲥"},
    {"鰥", "鳏"},
    {"鰨", "鳎"},
    {"鰩", "鳐"},
    {"鰭", "鳍"},
    {"鰱", "鲢"},
    {"鰲", "鳌"},
    {"鰳", "鳓"},
    {"鰷", "鲦"},
    {"鰹", "鲣"},
    {"鰻", "鳗"},
    {"鰾", "鳔"},
    {"鱈", "鳕"},
    {"鱉", "鳖"},
    {"鱒", "鳟"},
    {"鱔", "鳝"},
    {"鱖", "鳜"},
    {"鱗", "鳞"},
    {"鱘", "鲟"},
    {"鱟", "鲎"},
    {"鱧", "鳢"},
    {"鱭", "鲚"},
    {"鱷", "鳄"},
    {"鱸", "鲈"},
    {"鱺", "鲡"},
    {"鳥", "鸟"},
    {"鳧", "凫"},
    {"鳩", "鸠"},
    {"鳳", "凤"},
    {"鳴", "鸣"},
    {"鳶", "鸢"},
    {"鴆", "鸩"},
    {"鴇", "鸨"},
    {"鴈", "雁"},
    {"鴉", "鸦"},
    {"鴕", "鸵"},
    {"鴛", "鸳"},
    {"鴝", "鸲"},
    {"鴟", "鸱"},
    {"鴣", "鸪"},
    {"鴦", "鸯"},
    {"鴨", "鸭"},
    {"鴯", "鸸"},
    {"鴰", "鸹"},
    {"鴻", "鸿"},
    {"鴿", "鸽"},
    {"鵂", "鸺"},
    {"鵑", "鹃"},
    {"鵒", "鹆"},
    {"鵓", "鹁"},
    {"鵜", "鹈"},
    {"鵝", "鹅"},
    {"鵠", "鹄"},
    {"鵡", "鹉"},
    {"鵪", "鹌"},
    {"鵬", "鹏"},
    {"鵯", "鹎"},
    {"鵰", "雕"},
    {"鵲", "鹊"},
    {"鶇", "鸫"},
    {"鶉", "鹑"},
    {"鶘", "鹕"},
    {"鶚", "鹗"},
    {"鶩", "鹜"},
    {"鶯", "莺"},
    {"鶱", "骞"},
    {"鶴", "鹤"},
    {"鶻", "鹘"},
    {"鶼", "鹣"},
    {"鶿", "鹚"},
    {"鷂", "鹞"},
    {"鷓", "鹧"},
    {"鷗", "鸥"},
    {"鷙", "鸷"},
    {"鷚", "鹨"},
    {"鷥", "鸶"},
    {"鷦", "鹪"},
    {"鷯", "鹩"},
    {"鷰", "燕"},
    {"鷲", "鹫"},
    {"鷳", "鹇"},
    {"鷴", "鹇"},
    {"鷸", "鹬"},
    {"鷹", "鹰"},
    {"鷺", "鹭"},
    {"鸕", "鸬"},
    {"鸚", "鹦"},
    {"鸛", "鹳"},
    {"鸝", "鹂"},
    {"鸞", "鸾"},
    {"鹵", "卤"},
    {"鹹", "咸"},
    {"鹺", "鹾"},
    {"鹼", "硷"},
    {"鹽", "盐"},
    {"麗", "丽"},
    {"麥", "麦"},
    {"麩", "麸"},
    {"麵", "面"},
    {"麼", "么"},
    {"黃", "黄"},
    {"黌", "黉"},
    {"點", "点"},
    {"黨", "党"},
    {"黲", "黪"},
    {"黴", "霉"},
    {"黷", "黩"},
    {"黽", "黾"},
    {"黿", "鼋"},
    {"鼇", "鳌"},
    {"鼉", "鼍"},
    {"鼕", "冬"},
    {"鼴", "鼹"},
    {"齊", "齐"},
    {"齋", "斋"},
    {"齎", "赍"},
    {"齏", "齑"},
    {"齒", "齿"},
    {"齔", "龀"},
    {"齙", "龅"},
    {"齜", "龇"},
    {"齟", "龃"},
    {"齠", "龆"},
    {"齡", "龄"},
    {"齣", "出"},
    {"齦", "龈"},
    {"齧", "啮"},
    {"齪", "龊"},
    {"齬", "龉"},
    {"齲", "龋"},
    {"齶", "腭"},
    {"齷", "龌"},
    {"龍", "龙"},
    {"龐", "庞"},
    {"龔", "龚"},
    {"龕", "龛"},
    {"龜", "龟"},
    {"龢", "和"},
    {"兀", "兀"},
    {"︰", "∶"},
    {"︱", "｜"},
    {"︳", "｜"},
    {"︿", "∧"},
    {"﹀", "∨"},
    {"﹐", "，"},
    {"﹑", "、"},
    {"﹒", "．"},
    {"﹔", "；"},
    {"﹕", "："},
    {"﹖", "？"},
    {"﹗", "！"},
    {"﹙", "（"},
    {"﹚", "）"},
    {"﹛", "｛"},
    {"﹜", "｝"},
    {"﹝", "［"},
    {"﹞", "］"},
    {"﹟", "＃"},
    {"﹠", "＆"},
    {"﹡", "＊"},
    {"﹢", "＋"},
    {"﹣", "－"},
    {"﹤", "＜"},
    {"﹥", "＞"},
    {"﹦", "＝"},
    {"﹩", "＄"},
    {"﹪", "％"},
    {"﹫", "＠"},
};

static const ChineseCharacter gb2312_elements[] = {
    {"¨", "‥"},
    {"―", "─"},
    {"‖", "∥"},
    {"″", "〞"},
    {"∏", "Π"},
    {"∑", "Σ"},
    {"∧", "︿"},
    {"∨", "﹀"},
    {"∶", "︰"},
    {"≈", "≒"},
    {"≤", "≦"},
    {"≥", "≧"},
    {"━", "─"},
    {"┃", "│"},
    {"┏", "┌"},
    {"┓", "┐"},
    {"┗", "└"},
    {"┛", "┘"},
    {"┣", "├"},
    {"┫", "┤"},
    {"┳", "┬"},
    {"┻", "┴"},
    {"╋", "┼"},
    {"・", "·"},
    {"万", "萬"},
    {"与", "與"},
    {"丑", "醜"},
    {"专", "專"},
    {"业", "業"},
    {"丛", "叢"},
    {"东", "東"},
    {"丝", "絲"},
    {"丢", "丟"},
    {"两", "兩"},
    {"严", "嚴"},
    {"丧", "喪"},
    {"个", "個"},
    {"丬", "爿"},
    {"丰", "豐"},
    {"临", "臨"},
    {"为", "為"},
    {"丽", "麗"},
    {"举", "舉"},
    {"么", "麼"},
    {"义", "義"},
    {"乌", "烏"},
    {"乐", "樂"},
    {"乔", "喬"},
    {"习", "習"},
    {"乡", "鄉"},
    {"书", "書"},
    {"买", "買"},
    {"乱", "亂"},
    {"争", "爭"},
    {"亏", "虧"},
    {"云", "雲"},
    {"亘", "亙"},
    {"亚", "亞"},
    {"产", "產"},
    {"亩", "畝"},
    {"亲", "親"},
    {"亵", "褻"},
    {"亻", "人"},
    {"亿", "億"},
    {"仅", "僅"},
    {"仆", "僕"},
    {"从", "從"},
    {"仑", "侖"},
    {"仓", "倉"},
    {"仪", "儀"},
    {"们", "們"},
    {"价", "價"},
    {"众", "眾"},
    {"优", "優"},
    {"会", "會"},
    {"伛", "傴"},
    {"伞", "傘"},
    {"伟", "偉"},
    {"传", "傳"},
    {"伤", "傷"},
    {"伥", "倀"},
    {"伦", "倫"},
    {"伧", "傖"},
    {"伪", "偽"},
    {"伫", "佇"},
    {"伲", "你"},
    {"体", "體"},
    {"余", "餘"},
    {"佣", "傭"},
    {"佥", "僉"},
    {"侄", "姪"},
    {"侠", "俠"},
    {"侣", "侶"},
    {"侥", "僥"},
    {"侦", "偵"},
    {"侧", "側"},
    {"侨", "僑"},
    {"侩", "儈"},
    {"侪", "儕"},
    {"侬", "儂"},
    {"俣", "俁"},
    {"俦", "儔"},
    {"俨", "儼"},
    {"俩", "倆"},
    {"俪", "儷"},
    {"俭", "儉"},
    {"倮", "裸"},
    {"债", "債"},
    {"倾", "傾"},
    {"偬", "傯"},
    {"偻", "僂"},
    {"偾", "僨"},
    {"偿", "償"},
    {"傥", "儻"},
    {"傧", "儐"},
    {"储", "儲"},
    {"傩", "儺"},
    {"儿", "兒"},
    {"兑", "兌"},
    {"兖", "兗"},
    {"党", "黨"},
    {"兰", "蘭"},
    {"关", "關"},
    {"兴", "興"},
    {"兹", "茲"},
    {"养", "養"},
    {"兽", "獸"},
    {"冁", "囅"},
    {"内", "內"},
    {"冈", "岡"},
    {"册", "冊"},
    {"写", "寫"},
    {"军", "軍"},
    {"农", "農"},
    {"冢", "塚"},
    {"冯", "馮"},
    {"冲", "衝"},
    {"决", "決"},
    {"况", "況"},
    {"冻", "凍"},
    {"净", "淨"},
    {"凄", "淒"},
    {"准", "準"},
    {"凇", "淞"},
    {"凉", "涼"},
    {"减", "減"},
    {"凑", "湊"},
    {"凛", "凜"},
    {"几", "幾"},
    {"凤", "鳳"},
    {"凫", "鳧"},
    {"凭", "憑"},
    {"凯", "凱"},
    {"凶", "兇"},
    {"击", "擊"},
    {"凼", "幽"},
    {"凿", "鑿"},
    {"刂", "刀"},
    {"刍", "芻"},
    {"划", "劃"},
    {"刘", "劉"},
    {"则", "則"},
    {"刚", "剛"},
    {"创", "創"},
    {"删", "刪"},
    {"别", "別"},
    {"刭", "剄"},
    {"刹", "剎"},
    {"刽", "劊"},
    {"刿", "劌"},
    {"剀", "剴"},
    {"剂", "劑"},
    {"剐", "剮"},
    {"剑", "劍"},
    {"剥", "剝"},
    {"剧", "劇"},
    {"剿", "勦"},
    {"劝", "勸"},
    {"办", "辦"},
    {"务", "務"},
    {"劢", "勱"},
    {"动", "動"},
    {"励", "勵"},
    {"劲", "勁"},
    {"劳", "勞"},
    {"势", "勢"},
    {"勋", "勳"},
    {"勖", "勗"},
    {"匀", "勻"},
    {"匦", "匭"},
    {"匮", "匱"},
    {"区", "區"},
    {"医", "醫"},
    {"华", "華"},
    {"协", "協"},
    {"单", "單"},
    {"卖", "賣"},
    {"占", "佔"},
    {"卢", "盧"},
    {"卤", "滷"},
    {"卧", "臥"},
    {"卩", "部"},
    {"卫", "衛"},
    {"却", "卻"},
    {"卺", "巹"},
    {"厂", "廠"},
    {"厅", "廳"},
    {"历", "歷"},
    {"厉", "厲"},
    {"压", "壓"},
    {"厌", "厭"},
    {"厍", "厙"},
    {"厕", "廁"},
    {"厘", "釐"},
    {"厢", "廂"},
    {"厣", "厴"},
    {"厦", "廈"},
    {"厨", "廚"},
    {"厩", "廄"},
    {"厮", "廝"},
    {"厶", "私"},
    {"县", "縣"},
    {"叁", "參"},
    {"参", "參"},
    {"双", "雙"},
    {"发", "發"},
    {"变", "變"},
    {"叙", "敘"},
    {"叠", "疊"},
    {"叶", "葉"},
    {"号", "號"},
    {"叹", "嘆"},
    {"叽", "嘰"},
    {"吁", "籲"},
    {"后", "後"},
    {"吓", "嚇"},
    {"吕", "呂"},
    {"吗", "嗎"},
    {"吨", "噸"},
    {"听", "聽"},
    {"启", "啟"},
    {"吴", "吳"},
    {"呐", "吶"},
    {"呒", "嘸"},
    {"呓", "囈"},
    {"呕", "嘔"},
    {"呖", "嚦"},
    {"呗", "唄"},
    {"员", "員"},
    {"呙", "咼"},
    {"呛", "嗆"},
    {"呜", "嗚"},
    {"周", "週"},
    {"咏", "詠"},
    {"咙", "嚨"},
    {"咛", "嚀"},
    {"咸", "鹹"},
    {"哌", "呱"},
    {"响", "響"},
    {"哑", "啞"},
    {"哒", "噠"},
    {"哓", "嘵"},
    {"哔", "嗶"},
    {"哕", "噦"},
    {"哗", "嘩"},
    {"哙", "噲"},
    {"哜", "嚌"},
    {"哝", "噥"},
    {"哟", "喲"},
    {"唛", "嘜"},
    {"唠", "嘮"},
    {"唢", "嗩"},
    {"唤", "喚"},
    {"啧", "嘖"},
    {"啬", "嗇"},
    {"啭", "囀"},
    {"啮", "囓"},
    {"啸", "嘯"},
    {"喷", "噴"},
    {"喽", "嘍"},
    {"喾", "嚳"},
    {"嗫", "囁"},
    {"嗬", "呵"},
    {"嗳", "噯"},
    {"嘘", "噓"},
    {"嘤", "嚶"},
    {"嘱", "囑"},
    {"噜", "嚕"},
    {"嚣", "囂"},
    {"团", "團"},
    {"园", "園"},
    {"囱", "囪"},
    {"围", "圍"},
    {"囵", "圇"},
    {"国", "國"},
    {"图", "圖"},
    {"圆", "圓"},
    {"圣", "聖"},
    {"圹", "壙"},
    {"场", "場"},
    {"坂", "阪"},
    {"坏", "壞"},
    {"块", "塊"},
    {"坚", "堅"},
    {"坛", "壇"},
    {"坜", "壢"},
    {"坝", "壩"},
    {"坞", "塢"},
    {"坟", "墳"},
    {"坠", "墜"},
    {"垄", "壟"},
    {"垅", "壟"},
    {"垆", "壚"},
    {"垒", "壘"},
    {"垦", "墾"},
    {"垩", "堊"},
    {"垫", "墊"},
    {"垭", "埡"},
    {"垲", "塏"},
    {"埘", "塒"},
    {"埙", "壎"},
    {"埚", "堝"},
    {"堑", "塹"},
    {"堕", "墮"},
    {"墒", "墑"},
    {"墙", "牆"},
    {"壮", "壯"},
    {"声", "聲"},
    {"壳", "殼"},
    {"壶", "壺"},
    {"处", "處"},
    {"备", "備"},
    {"复", "復"},
    {"够", "夠"},
    {"头", "頭"},
    {"夸", "誇"},
    {"夹", "夾"},
    {"夺", "奪"},
    {"奁", "奩"},
    {"奂", "奐"},
    {"奋", "奮"},
    {"奖", "獎"},
    {"奥", "奧"},
    {"妆", "妝"},
    {"妇", "婦"},
    {"妈", "媽"},
    {"妩", "嫵"},
    {"妪", "嫗"},
    {"妫", "媯"},
    {"姗", "姍"},
    {"娄", "婁"},
    {"娅", "婭"},
    {"娆", "嬈"},
    {"娇", "嬌"},
    {"娈", "孌"},
    {"娱", "娛"},
    {"娲", "媧"},
    {"娴", "嫻"},
    {"婴", "嬰"},
    {"婵", "嬋"},
    {"婶", "嬸"},
    {"媪", "媼"},
    {"嫒", "嬡"},
    {"嫔", "嬪"},
    {"嫱", "嬙"},
    {"嬷", "嬤"},
    {"孙", "孫"},
    {"学", "學"},
    {"孪", "孿"},
    {"宁", "寧"},
    {"宝", "寶"},
    {"实", "實"},
    {"宠", "寵"},
    {"审", "審"},
    {"宪", "憲"},
    {"宫", "宮"},
    {"宽", "寬"},
    {"宾", "賓"},
    {"寝", "寢"},
    {"对", "對"},
    {"寻", "尋"},
    {"导", "導"},
    {"寿", "壽"},
    {"将", "將"},
    {"尔", "爾"},
    {"尘", "塵"},
    {"尜", "嘎"},
    {"尝", "嘗"},
    {"尧", "堯"},
    {"尴", "尷"},
    {"尸", "屍"},
    {"尽", "盡"},
    {"层", "層"},
    {"屉", "屜"},
    {"届", "屆"},
    {"属", "屬"},
    {"屡", "屢"},
    {"屦", "屨"},
    {"屿", "嶼"},
    {"岁", "歲"},
    {"岂", "豈"},
    {"岖", "嶇"},
    {"岗", "崗"},
    {"岘", "峴"},
    {"岚", "嵐"},
    {"岛", "島"},
    {"岭", "嶺"},
    {"岽", "崠"},
    {"岿", "巋"},
    {"峄", "嶧"},
    {"峡", "峽"},
    {"峤", "嶠"},
    {"峥", "崢"},
    {"峦", "巒"},
    {"崂", "嶗"},
    {"崃", "崍"},
    {"崭", "嶄"},
    {"嵘", "嶸"},
    {"嵛", "崳"},
    {"嵝", "嶁"},
    {"巅", "巔"},
    {"巩", "鞏"},
    {"巯", "巰"},
    {"币", "幣"},
    {"帅", "帥"},
    {"师", "師"},
    {"帏", "幃"},
    {"帐", "帳"},
    {"帘", "簾"},
    {"帜", "幟"},
    {"带", "帶"},
    {"帧", "幀"},
    {"帮", "幫"},
    {"帱", "幬"},
    {"帻", "幘"},
    {"帼", "幗"},
    {"幂", "冪"},
    {"干", "幹"},
    {"并", "並"},
    {"幺", "么"},
    {"广", "廣"},
    {"庄", "莊"},
    {"庆", "慶"},
    {"庐", "廬"},
    {"庑", "廡"},
    {"库", "庫"},
    {"应", "應"},
    {"庙", "廟"},
    {"庞", "龐"},
    {"废", "廢"},
    {"廪", "廩"},
    {"开", "開"},
    {"异", "異"},
    {"弃", "棄"},
    {"弑", "弒"},
    {"张", "張"},
    {"弥", "彌"},
    {"弪", "弳"},
    {"弯", "彎"},
    {"弹", "彈"},
    {"强", "強"},
    {"归", "歸"},
    {"当", "當"},
    {"录", "錄"},
    {"彦", "彥"},
    {"彻", "徹"},
    {"径", "徑"},
    {"徕", "徠"},
    {"御", "禦"},
    {"忄", "心"},
    {"忆", "憶"},
    {"忏", "懺"},
    {"忧", "憂"},
    {"忾", "愾"},
    {"怀", "懷"},
    {"态", "態"},
    {"怂", "慫"},
    {"怃", "憮"},
    {"怄", "慪"},
    {"怅", "悵"},
    {"怆", "愴"},
    {"怜", "憐"},
    {"总", "總"},
    {"怼", "懟"},
    {"怿", "懌"},
    {"恋", "戀"},
    {"恒", "恆"},
    {"恳", "懇"},
    {"恶", "惡"},
    {"恸", "慟"},
    {"恹", "懨"},
    {"恺", "愷"},
    {"恻", "惻"},
    {"恼", "惱"},
    {"恽", "惲"},
    {"悦", "悅"},
    {"悫", "愨"},
    {"悬", "懸"},
    {"悭", "慳"},
    {"悯", "憫"},
    {"惊", "驚"},
    {"惧", "懼"},
    {"惨", "慘"},
    {"惩", "懲"},
    {"惫", "憊"},
    {"惬", "愜"},
    {"惭", "慚"},
    {"惮", "憚"},
    {"惯", "慣"},
    {"愠", "慍"},
    {"愤", "憤"},
    {"愦", "憒"},
    {"愿", "願"},
    {"慑", "懾"},
    {"懑", "懣"},
    {"懒", "懶"},
    {"懔", "懍"},
    {"戆", "戇"},
    {"戋", "戔"},
    {"戏", "戲"},
    {"戗", "戧"},
    {"战", "戰"},
    {"戬", "戩"},
    {"户", "戶"},
    {"扌", "手"},
    {"扑", "撲"},
    {"托", "託"},
    {"执", "執"},
    {"扩", "擴"},
    {"扪", "捫"},
    {"扫", "掃"},
    {"扬", "揚"},
    {"扰", "擾"},
    {"抚", "撫"},
    {"抛", "拋"},
    {"抟", "摶"},
    {"抠", "摳"},
    {"抡", "掄"},
    {"抢", "搶"},
    {"护", "護"},
    {"报", "報"},
    {"担", "擔"},
    {"拟", "擬"},
    {"拢", "攏"},
    {"拣", "揀"},
    {"拥", "擁"},
    {"拦", "攔"},
    {"拧", "擰"},
    {"拨", "撥"},
    {"择", "擇"},
    {"挂", "掛"},
    {"挚", "摯"},
    {"挛", "攣"},
    {"挝", "撾"},
    {"挞", "撻"},
    {"挟", "挾"},
    {"挠", "撓"},
    {"挡", "擋"},
    {"挢", "撟"},
    {"挣", "掙"},
    {"挤", "擠"},
    {"挥", "揮"},
    {"捂", "摀"},
    {"捞", "撈"},
    {"损", "損"},
    {"捡", "撿"},
    {"换", "換"},
    {"捣", "搗"},
    {"据", "據"},
    {"掳", "擄"},
    {"掴", "摑"},
    {"掷", "擲"},
    {"掸", "撢"},
    {"掺", "摻"},
    {"掼", "摜"},
    {"揽", "攬"},
    {"揿", "撳"},
    {"搀", "攙"},
    {"搁", "擱"},
    {"搂", "摟"},
    {"搅", "攪"},
    {"携", "攜"},
    {"摄", "攝"},
    {"摅", "攄"},
    {"摆", "擺"},
    {"摇", "搖"},
    {"摈", "擯"},
    {"摊", "攤"},
    {"撄", "攖"},
    {"撑", "撐"},
    {"撵", "攆"},
    {"撷", "擷"},
    {"撸", "擼"},
    {"撺", "攛"},
    {"擀", "搟"},
    {"擞", "擻"},
    {"攒", "攢"},
    {"攴", "撲"},
    {"敌", "敵"},
    {"敛", "斂"},
    {"数", "數"},
    {"斋", "齋"},
    {"斓", "斕"},
    {"斗", "鬥"},
    {"斩", "斬"},
    {"断", "斷"},
    {"无", "無"},
    {"旧", "舊"},
    {"时", "時"},
    {"旷", "曠"},
    {"昙", "曇"},
    {"昵", "暱"},
    {"昼", "晝"},
    {"显", "顯"},
    {"晋", "晉"},
    {"晒", "曬"},
    {"晓", "曉"},
    {"晔", "曄"},
    {"晕", "暈"},
    {"晖", "暉"},
    {"暂", "暫"},
    {"暧", "曖"},
    {"朐", "胊"},
    {"术", "術"},
    {"朴", "樸"},
    {"机", "機"},
    {"杀", "殺"},
    {"杂", "雜"},
    {"权", "權"},
    {"杆", "桿"},
    {"杠", "槓"},
    {"条", "條"},
    {"来", "來"},
    {"杨", "楊"},
    {"杩", "榪"},
    {"杰", "傑"},
    {"松", "鬆"},
    {"极", "極"},
    {"构", "構"},
    {"枞", "樅"},
    {"枢", "樞"},
    {"枣", "棗"},
    {"枥", "櫪"},
    {"枨", "棖"},
    {"枪", "槍"},
    {"枫", "楓"},
    {"枭", "梟"},
    {"柜", "櫃"},
    {"柠", "檸"},
    {"柽", "檉"},
    {"栀", "梔"},
    {"栅", "柵"},
    {"标", "標"},
    {"栈", "棧"},
    {"栉", "櫛"},
    {"栊", "櫳"},
    {"栋", "棟"},
    {"栌", "櫨"},
    {"栎", "櫟"},
    {"栏", "欄"},
    {"树", "樹"},
    {"栖", "棲"},
    {"样", "樣"},
    {"栾", "欒"},
    {"桠", "椏"},
    {"桡", "橈"},
    {"桢", "楨"},
    {"档", "檔"},
    {"桤", "榿"},
    {"桥", "橋"},
    {"桦", "樺"},
    {"桧", "檜"},
    {"桨", "槳"},
    {"桩", "樁"},
    {"梦", "夢"},
    {"检", "檢"},
    {"棂", "櫺"},
    {"棰", "箠"},
    {"棱", "稜"},
    {"椁", "槨"},
    {"椟", "櫝"},
    {"椠", "槧"},
    {"椤", "欏"},
    {"椭", "橢"},
    {"楼", "樓"},
    {"榄", "欖"},
    {"榇", "櫬"},
    {"榈", "櫚"},
    {"榉", "櫸"},
    {"榘", "矩"},
    {"槛", "檻"},
    {"槟", "檳"},
    {"槠", "櫧"},
    {"横", "橫"},
    {"樯", "檣"},
    {"樱", "櫻"},
    {"橥", "櫫"},
    {"橱", "櫥"},
    {"橹", "櫓"},
    {"橼", "櫞"},
    {"檐", "簷"},
    {"檩", "檁"},
    {"欢", "歡"},
    {"欤", "歟"},
    {"欧", "歐"},
    {"歼", "殲"},
    {"殁", "歿"},
    {"殇", "殤"},
    {"残", "殘"},
    {"殒", "殞"},
    {"殓", "殮"},
    {"殚", "殫"},
    {"殡", "殯"},
    {"殴", "毆"},
    {"毁", "毀"},
    {"毂", "轂"},
    {"毕", "畢"},
    {"毙", "斃"},
    {"毡", "氈"},
    {"毵", "毿"},
    {"氇", "氌"},
    {"气", "氣"},
    {"氢", "氫"},
    {"氩", "氬"},
    {"氲", "氳"},
    {"氵", "水"},
    {"氽", "汆"},
    {"汇", "匯"},
    {"汉", "漢"},
    {"汤", "湯"},
    {"汹", "洶"},
    {"沟", "溝"},
    {"没", "沒"},
    {"沣", "灃"},
    {"沤", "漚"},
    {"沥", "瀝"},
    {"沦", "淪"},
    {"沧", "滄"},
    {"沩", "溈"},
    {"沪", "滬"},
    {"沲", "沱"},
    {"泄", "洩"},
    {"泞", "濘"},
    {"泪", "淚"},
    {"泶", "澩"},
    {"泷", "瀧"},
    {"泸", "瀘"},
    {"泺", "濼"},
    {"泻", "瀉"},
    {"泼", "潑"},
    {"泽", "澤"},
    {"泾", "涇"},
    {"洁", "潔"},
    {"洒", "灑"},
    {"洼", "窪"},
    {"浃", "浹"},
    {"浅", "淺"},
    {"浆", "漿"},
    {"浇", "澆"},
    {"浈", "湞"},
    {"浊", "濁"},
    {"测", "測"},
    {"浍", "澮"},
    {"济", "濟"},
    {"浏", "瀏"},
    {"浑", "渾"},
    {"浒", "滸"},
    {"浓", "濃"},
    {"浔", "潯"},
    {"浜", "濱"},
    {"涂", "塗"},
    {"涌", "湧"},
    {"涛", "濤"},
    {"涝", "澇"},
    {"涞", "淶"},
    {"涟", "漣"},
    {"涠", "潿"},
    {"涡", "渦"},
    {"涣", "渙"},
    {"涤", "滌"},
    {"润", "潤"},
    {"涧", "澗"},
    {"涨", "漲"},
    {"涩", "澀"},
    {"淀", "澱"},
    {"渊", "淵"},
    {"渌", "淥"},
    {"渍", "漬"},
    {"渎", "瀆"},
    {"渐", "漸"},
    {"渑", "澠"},
    {"渔", "漁"},
    {"渖", "瀋"},
    {"渗", "滲"},
    {"温", "溫"},
    {"游", "遊"},
    {"湾", "灣"},
    {"湿", "濕"},
    {"溃", "潰"},
    {"溅", "濺"},
    {"溆", "漵"},
    {"滗", "潷"},
    {"滚", "滾"},
    {"滞", "滯"},
    {"滟", "灩"},
    {"滠", "灄"},
    {"满", "滿"},
    {"滢", "瀅"},
    {"滤", "濾"},
    {"滥", "濫"},
    {"滦", "灤"},
    {"滨", "濱"},
    {"滩", "灘"},
    {"潆", "瀠"},
    {"潇", "瀟"},
    {"潋", "瀲"},
    {"潍", "濰"},
    {"潜", "潛"},
    {"潴", "瀦"},
    {"澜", "瀾"},
    {"濑", "瀨"},
    {"濒", "瀕"},
    {"灏", "灝"},
    {"灬", "火"},
    {"灭", "滅"},
    {"灯", "燈"},
    {"灵", "靈"},
    {"灾", "災"},
    {"灿", "燦"},
    {"炀", "煬"},
    {"炉", "爐"},
    {"炖", "燉"},
    {"炜", "煒"},
    {"炝", "熗"},
    {"炮", "砲"},
    {"点", "點"},
    {"炼", "鍊"},
    {"炽", "熾"},
    {"烁", "爍"},
    {"烂", "爛"},
    {"烃", "烴"},
    {"烛", "燭"},
    {"烟", "煙"},
    {"烦", "煩"},
    {"烧", "燒"},
    {"烨", "燁"},
    {"烩", "燴"},
    {"烫", "燙"},
    {"烬", "燼"},
    {"热", "熱"},
    {"焕", "煥"},
    {"焖", "燜"},
    {"焘", "燾"},
    {"焰", "燄"},
    {"煅", "鍛"},
    {"熏", "燻"},
    {"爱", "愛"},
    {"爷", "爺"},
    {"牍", "牘"},
    {"牦", "犛"},
    {"牵", "牽"},
    {"牺", "犧"},
    {"犊", "犢"},
    {"犭", "犬"},
    {"状", "狀"},
    {"犷", "獷"},
    {"犹", "猶"},
    {"狈", "狽"},
    {"狞", "獰"},
    {"独", "獨"},
    {"狭", "狹"},
    {"狮", "獅"},
    {"狯", "獪"},
    {"狰", "猙"},
    {"狱", "獄"},
    {"狲", "猻"},
    {"猃", "獫"},
    {"猎", "獵"},
    {"猕", "獼"},
    {"猡", "玀"},
    {"猪", "豬"},
    {"猫", "貓"},
    {"猬", "蝟"},
    {"献", "獻"},
    {"獭", "獺"},
    {"玑", "璣"},
    {"玛", "瑪"},
    {"玮", "瑋"},
    {"环", "環"},
    {"现", "現"},
    {"玺", "璽"},
    {"珉", "玟"},
    {"珏", "玨"},
    {"珐", "琺"},
    {"珑", "瓏"},
    {"珲", "琿"},
    {"琏", "璉"},
    {"琐", "瑣"},
    {"琼", "瓊"},
    {"瑶", "瑤"},
    {"瑷", "璦"},
    {"璎", "瓔"},
    {"瓒", "瓚"},
    {"瓮", "甕"},
    {"瓯", "甌"},
    {"电", "電"},
    {"画", "畫"},
    {"畅", "暢"},
    {"畲", "畬"},
    {"畴", "疇"},
    {"疖", "癤"},
    {"疗", "療"},
    {"疟", "瘧"},
    {"疠", "癘"},
    {"疡", "瘍"},
    {"疮", "瘡"},
    {"疯", "瘋"},
    {"疱", "皰"},
    {"疴", "痾"},
    {"痈", "癰"},
    {"痉", "痙"},
    {"痒", "癢"},
    {"痨", "癆"},
    {"痪", "瘓"},
    {"痫", "癇"},
    {"痴", "癡"},
    {"痹", "痺"},
    {"瘅", "癉"},
    {"瘗", "瘞"},
    {"瘘", "瘺"},
    {"瘪", "癟"},
    {"瘫", "癱"},
    {"瘾", "癮"},
    {"瘿", "癭"},
    {"癞", "癩"},
    {"癣", "癬"},
    {"癫", "癲"},
    {"皑", "皚"},
    {"皱", "皺"},
    {"皲", "皸"},
    {"盏", "盞"},
    {"盐", "鹽"},
    {"监", "監"},
    {"盖", "蓋"},
    {"盗", "盜"},
    {"盘", "盤"},
    {"眦", "眥"},
    {"眯", "瞇"},
    {"着", "著"},
    {"睁", "睜"},
    {"睐", "睞"},
    {"睑", "瞼"},
    {"睾", "睪"},
    {"睿", "叡"},
    {"瞒", "瞞"},
    {"瞩", "矚"},
    {"矫", "矯"},
    {"矶", "磯"},
    {"矾", "礬"},
    {"矿", "礦"},
    {"砀", "碭"},
    {"码", "碼"},
    {"砖", "磚"},
    {"砗", "硨"},
    {"砚", "硯"},
    {"砺", "礪"},
    {"砻", "礱"},
    {"砾", "礫"},
    {"础", "礎"},
    {"硕", "碩"},
    {"硖", "硤"},
    {"硗", "磽"},
    {"确", "確"},
    {"硷", "鹼"},
    {"碍", "礙"},
    {"碛", "磧"},
    {"碜", "磣"},
    {"碱", "堿"},
    {"礴", "礡"},
    {"礻", "示"},
    {"礼", "禮"},
    {"祛", "袪"},
    {"祢", "禰"},
    {"祯", "禎"},
    {"祷", "禱"},
    {"祸", "禍"},
    {"禀", "稟"},
    {"禄", "祿"},
    {"禅", "禪"},
    {"离", "離"},
    {"秃", "禿"},
    {"秆", "稈"},
    {"种", "種"},
    {"积", "積"},
    {"称", "稱"},
    {"秽", "穢"},
    {"税", "稅"},
    {"稣", "穌"},
    {"稳", "穩"},
    {"穑", "穡"},
    {"穷", "窮"},
    {"窃", "竊"},
    {"窍", "竅"},
    {"窑", "窯"},
    {"窜", "竄"},
    {"窝", "窩"},
    {"窥", "窺"},
    {"窦", "竇"},
    {"窭", "窶"},
    {"竖", "豎"},
    {"竞", "競"},
    {"笃", "篤"},
    {"笋", "筍"},
    {"笔", "筆"},
    {"笕", "筧"},
    {"笺", "箋"},
    {"笼", "籠"},
    {"笾", "籩"},
    {"筇", "笻"},
    {"筑", "築"},
    {"筚", "篳"},
    {"筛", "篩"},
    {"筝", "箏"},
    {"筱", "篠"},
    {"筹", "籌"},
    {"签", "簽"},
    {"简", "簡"},
    {"箦", "簀"},
    {"箧", "篋"},
    {"箨", "籜"},
    {"箩", "籮"},
    {"箪", "簞"},
    {"箫", "簫"},
    {"篑", "簣"},
    {"篓", "簍"},
    {"篮", "籃"},
    {"篱", "籬"},
    {"簖", "籪"},
    {"籁", "籟"},
    {"籴", "糴"},
    {"类", "類"},
    {"籼", "秈"},
    {"粜", "糶"},
    {"粝", "糲"},
    {"粤", "粵"},
    {"粪", "糞"},
    {"粮", "糧"},
    {"糁", "糝"},
    {"糇", "餱"},
    {"糍", "餈"},
    {"紧", "緊"},
    {"絷", "縶"},
    {"纟", "絲"},
    {"纠", "糾"},
    {"纡", "紆"},
    {"红", "紅"},
    {"纣", "紂"},
    {"纤", "纖"},
    {"纥", "紇"},
    {"约", "約"},
    {"级", "級"},
    {"纨", "紈"},
    {"纩", "纊"},
    {"纪", "紀"},
    {"纫", "紉"},
    {"纬", "緯"},
    {"纭", "紜"},
    {"纯", "純"},
    {"纰", "紕"},
    {"纱", "紗"},
    {"纲", "綱"},
    {"纳", "納"},
    {"纵", "縱"},
    {"纶", "綸"},
    {"纷", "紛"},
    {"纸", "紙"},
    {"纹", "紋"},
    {"纺", "紡"},
    {"纽", "紐"},
    {"纾", "紓"},
    {"线", "線"},
    {"绀", "紺"},
    {"绁", "紲"},
    {"绂", "紱"},
    {"练", "練"},
    {"组", "組"},
    {"绅", "紳"},
    {"细", "細"},
    {"织", "織"},
    {"终", "終"},
    {"绉", "縐"},
    {"绊", "絆"},
    {"绋", "紼"},
    {"绌", "絀"},
    {"绍", "紹"},
    {"绎", "繹"},
    {"经", "經"},
    {"绐", "紿"},
    {"绑", "綁"},
    {"绒", "絨"},
    {"结", "結"},
    {"绔", "褲"},
    {"绕", "繞"},
    {"绗", "絎"},
    {"绘", "繪"},
    {"给", "給"},
    {"绚", "絢"},
    {"绛", "絳"},
    {"络", "絡"},
    {"绝", "絕"},
    {"绞", "絞"},
    {"统", "統"},
    {"绠", "綆"},
    {"绡", "綃"},
    {"绢", "絹"},
    {"绣", "繡"},
    {"绥", "綏"},
    {"绦", "絛"},
    {"继", "繼"},
    {"绨", "綈"},
    {"绩", "績"},
    {"绪", "緒"},
    {"绫", "綾"},
    {"续", "續"},
    {"绮", "綺"},
    {"绯", "緋"},
    {"绰", "綽"},
    {"绲", "緄"},
    {"绳", "繩"},
    {"维", "維"},
    {"绵", "綿"},
    {"绶", "綬"},
    {"绷", "繃"},
    {"绸", "綢"},
    {"绺", "綹"},
    {"绻", "綣"},
    {"综", "綜"},
    {"绽", "綻"},
    {"绾", "綰"},
    {"绿", "綠"},
    {"缀", "綴"},
    {"缁", "緇"},
    {"缂", "緙"},
    {"缃", "緗"},
    {"缄", "緘"},
    {"缅", "緬"},
    {"缆", "纜"},
    {"缇", "緹"},
    {"缈", "緲"},
    {"缉", "緝"},
    {"缋", "繢"},
    {"缌", "緦"},
    {"缍", "綞"},
    {"缎", "緞"},
    {"缏", "緶"},
    {"缑", "緱"},
    {"缒", "縋"},
    {"缓", "緩"},
    {"缔", "締"},
    {"缕", "縷"},
    {"编", "編"},
    {"缗", "緡"},
    {"缘", "緣"},
    {"缙", "縉"},
    {"缚", "縛"},
    {"缛", "縟"},
    {"缜", "縝"},
    {"缝", "縫"},
    {"缟", "縞"},
    {"缠", "纏"},
    {"缡", "縭"},
    {"缢", "縊"},
    {"缣", "縑"},
    {"缤", "繽"},
    {"缥", "縹"},
    {"缦", "縵"},
    {"缧", "縲"},
    {"缨", "纓"},
    {"缩", "縮"},
    {"缪", "繆"},
    {"缫", "繅"},
    {"缬", "纈"},
    {"缭", "繚"},
    {"缮", "繕"},
    {"缯", "繒"},
    {"缰", "韁"},
    {"缱", "繾"},
    {"缲", "繰"},
    {"缳", "繯"},
    {"缴", "繳"},
    {"缵", "纘"},
    {"罂", "罌"},
    {"网", "網"},
    {"罗", "羅"},
    {"罚", "罰"},
    {"罢", "罷"},
    {"罴", "羆"},
    {"羁", "羈"},
    {"羟", "羥"},
    {"羡", "羨"},
    {"翘", "翹"},
    {"耧", "耬"},
    {"耸", "聳"},
    {"耻", "恥"},
    {"聂", "聶"},
    {"聋", "聾"},
    {"职", "職"},
    {"聍", "聹"},
    {"联", "聯"},
    {"聩", "聵"},
    {"聪", "聰"},
    {"肀", "聿"},
    {"肃", "肅"},
    {"肠", "腸"},
    {"肤", "膚"},
    {"肮", "骯"},
    {"肴", "餚"},
    {"肾", "腎"},
    {"肿", "腫"},
    {"胀", "脹"},
    {"胁", "脅"},
    {"胄", "冑"},
    {"胆", "膽"},
    {"胜", "勝"},
    {"胧", "朧"},
    {"胪", "臚"},
    {"胫", "脛"},
    {"胶", "膠"},
    {"脉", "脈"},
    {"脍", "膾"},
    {"脏", "髒"},
    {"脐", "臍"},
    {"脑", "腦"},
    {"脓", "膿"},
    {"脔", "臠"},
    {"脚", "腳"},
    {"脱", "脫"},
    {"脲", "尿"},
    {"脶", "腡"},
    {"脸", "臉"},
    {"腊", "臘"},
    {"腌", "醃"},
    {"腭", "齶"},
    {"腻", "膩"},
    {"腼", "靦"},
    {"腽", "膃"},
    {"腾", "騰"},
    {"膑", "臏"},
    {"舆", "輿"},
    {"舣", "艤"},
    {"舰", "艦"},
    {"舱", "艙"},
    {"舻", "艫"},
    {"艰", "艱"},
    {"艳", "豔"},
    {"艹", "艸"},
    {"艺", "藝"},
    {"节", "節"},
    {"芈", "羋"},
    {"芗", "薌"},
    {"芜", "蕪"},
    {"芦", "蘆"},
    {"苁", "蓯"},
    {"苇", "葦"},
    {"苈", "藶"},
    {"苋", "莧"},
    {"苌", "萇"},
    {"苍", "蒼"},
    {"苎", "苧"},
    {"苏", "蘇"},
    {"苹", "蘋"},
    {"范", "範"},
    {"茎", "莖"},
    {"茏", "蘢"},
    {"茑", "蔦"},
    {"茔", "塋"},
    {"茕", "煢"},
    {"茧", "繭"},
    {"荆", "荊"},
    {"荐", "薦"},
    {"荚", "莢"},
    {"荛", "蕘"},
    {"荜", "蓽"},
    {"荞", "蕎"},
    {"荟", "薈"},
    {"荠", "薺"},
    {"荡", "蕩"},
    {"荣", "榮"},
    {"荤", "葷"},
    {"荥", "滎"},
    {"荦", "犖"},
    {"荧", "熒"},
    {"荨", "蕁"},
    {"荩", "藎"},
    {"荪", "蓀"},
    {"荫", "蔭"},
    {"荬", "藚"},
    {"荭", "葒"},
    {"药", "藥"},
    {"莅", "蒞"},
    {"莱", "萊"},
    {"莲", "蓮"},
    {"莳", "蒔"},
    {"莴", "萵"},
    {"莶", "薟"},
    {"获", "獲"},
    {"莸", "蕕"},
    {"莹", "瑩"},
    {"莺", "鶯"},
    {"莼", "蓴"},
    {"萝", "蘿"},
    {"萤", "螢"},
    {"营", "營"},
    {"萦", "縈"},
    {"萧", "蕭"},
    {"萨", "薩"},
    {"葱", "蔥"},
    {"蒇", "蕆"},
    {"蒉", "蕢"},
    {"蒋", "蔣"},
    {"蒌", "蔞"},
    {"蓑", "簑"},
    {"蓝", "藍"},
    {"蓟", "薊"},
    {"蓠", "蘺"},
    {"蓣", "蕷"},
    {"蓥", "鎣"},
    {"蓦", "驀"},
    {"蔷", "薔"},
    {"蔹", "蘞"},
    {"蔺", "藺"},
    {"蔼", "藹"},
    {"蕲", "蘄"},
    {"蕴", "蘊"},
    {"薮", "藪"},
    {"藓", "蘚"},
    {"蘖", "蘗"},
    {"虏", "虜"},
    {"虑", "慮"},
    {"虚", "虛"},
    {"虫", "蟲"},
    {"虬", "虯"},
    {"虮", "蟣"},
    {"虽", "雖"},
    {"虾", "蝦"},
    {"虿", "蠆"},
    {"蚀", "蝕"},
    {"蚁", "蟻"},
    {"蚂", "螞"},
    {"蚕", "蠶"},
    {"蚝", "蠔"},
    {"蚬", "蜆"},
    {"蛊", "蠱"},
    {"蛎", "蠣"},
    {"蛏", "蟶"},
    {"蛮", "蠻"},
    {"蛰", "蟄"},
    {"蛱", "蛺"},
    {"蛲", "蟯"},
    {"蛳", "螄"},
    {"蛴", "蠐"},
    {"蜕", "蛻"},
    {"蜗", "蝸"},
    {"蜡", "蠟"},
    {"蝇", "蠅"},
    {"蝈", "蟈"},
    {"蝉", "蟬"},
    {"蝎", "蠍"},
    {"蝰", "虺"},
    {"蝼", "螻"},
    {"蝾", "蠑"},
    {"蟮", "蟺"},
    {"衅", "釁"},
    {"衔", "銜"},
    {"衤", "衣"},
    {"补", "補"},
    {"衬", "襯"},
    {"衮", "袞"},
    {"袄", "襖"},
    {"袅", "裊"},
    {"袜", "襪"},
    {"袭", "襲"},
    {"装", "裝"},
    {"裆", "襠"},
    {"裢", "褳"},
    {"裣", "襝"},
    {"裤", "褲"},
    {"裥", "襉"},
    {"褛", "褸"},
    {"褴", "襤"},
    {"见", "見"},
    {"观", "觀"},
    {"规", "規"},
    {"觅", "覓"},
    {"视", "視"},
    {"觇", "覘"},
    {"览", "覽"},
    {"觉", "覺"},
    {"觊", "覬"},
    {"觋", "覡"},
    {"觌", "覿"},
    {"觎", "覦"},
    {"觏", "覯"},
    {"觐", "覲"},
    {"觑", "覷"},
    {"觞", "觴"},
    {"触", "觸"},
    {"觯", "觶"},
    {"誉", "譽"},
    {"誊", "謄"},
    {"讠", "言"},
    {"计", "計"},
    {"订", "訂"},
    {"讣", "訃"},
    {"认", "認"},
    {"讥", "譏"},
    {"讦", "訐"},
    {"讧", "訌"},
    {"讨", "討"},
    {"让", "讓"},
    {"讪", "訕"},
    {"讫", "訖"},
    {"训", "訓"},
    {"议", "議"},
    {"讯", "訊"},
    {"记", "記"},
    {"讲", "講"},
    {"讳", "諱"},
    {"讴", "謳"},
    {"讵", "詎"},
    {"讶", "訝"},
    {"讷", "訥"},
    {"许", "許"},
    {"讹", "訛"},
    {"论", "論"},
    {"讼", "訟"},
    {"讽", "諷"},
    {"设", "設"},
    {"访", "訪"},
    {"诀", "訣"},
    {"证", "證"},
    {"诂", "詁"},
    {"诃", "訶"},
    {"评", "評"},
    {"诅", "詛"},
    {"识", "識"},
    {"诈", "詐"},
    {"诉", "訴"},
    {"诊", "診"},
    {"诋", "詆"},
    {"诌", "謅"},
    {"词", "詞"},
    {"诎", "詘"},
    {"诏", "詔"},
    {"译", "譯"},
    {"诒", "詒"},
    {"诓", "誆"},
    {"诔", "誄"},
    {"试", "試"},
    {"诖", "詿"},
    {"诗", "詩"},
    {"诘", "詰"},
    {"诙", "詼"},
    {"诚", "誠"},
    {"诛", "誅"},
    {"诜", "詵"},
    {"话", "話"},
    {"诞", "誕"},
    {"诟", "詬"},
    {"诠", "詮"},
    {"诡", "詭"},
    {"询", "詢"},
    {"诣", "詣"},
    {"诤", "諍"},
    {"该", "該"},
    {"详", "詳"},
    {"诧", "詫"},
    {"诨", "諢"},
    {"诩", "詡"},
    {"诫", "誡"},
    {"诬", "誣"},
    {"语", "語"},
    {"诮", "誚"},
    {"误", "誤"},
    {"诰", "誥"},
    {"诱", "誘"},
    {"诲", "誨"},
    {"诳", "誑"},
    {"说", "說"},
    {"诵", "誦"},
    {"诶", "誒"},
    {"请", "請"},
    {"诸", "諸"},
    {"诹", "諏"},
    {"诺", "諾"},
    {"读", "讀"},
    {"诼", "諑"},
    {"诽", "誹"},
    {"课", "課"},
    {"诿", "諉"},
    {"谀", "諛"},
    {"谁", "誰"},
    {"谂", "諗"},
    {"调", "調"},
    {"谄", "諂"},
    {"谅", "諒"},
    {"谆", "諄"},
    {"谇", "誶"},
    {"谈", "談"},
    {"谊", "誼"},
    {"谋", "謀"},
    {"谌", "諶"},
    {"谍", "諜"},
    {"谎", "謊"},
    {"谏", "諫"},
    {"谐", "諧"},
    {"谑", "謔"},
    {"谒", "謁"},
    {"谓", "謂"},
    {"谔", "諤"},
    {"谕", "諭"},
    {"谖", "諼"},
    {"谗", "讒"},
    {"谘", "諮"},
    {"谙", "諳"},
    {"谚", "諺"},
    {"谛", "諦"},
    {"谜", "謎"},
    {"谝", "諞"},
    {"谟", "謨"},
    {"谠", "讜"},
    {"谡", "謖"},
    {"谢", "謝"},
    {"谣", "謠"},
    {"谤", "謗"},
    {"谥", "謚"},
    {"谦", "謙"},
    {"谧", "謐"},
    {"谨", "謹"},
    {"谩", "謾"},
    {"谪", "謫"},
    {"谫", "譾"},
    {"谬", "謬"},
    {"谭", "譚"},
    {"谮", "譖"},
    {"谯", "譙"},
    {"谰", "讕"},
    {"谱", "譜"},
    {"谲", "譎"},
    {"谳", "讞"},
    {"谴", "譴"},
    {"谵", "譫"},
    {"谶", "讖"},
    {"贝", "貝"},
    {"贞", "貞"},
    {"负", "負"},
    {"贡", "貢"},
    {"财", "財"},
    {"责", "責"},
    {"贤", "賢"},
    {"败", "敗"},
    {"账", "賬"},
    {"货", "貨"},
    {"质", "質"},
    {"贩", "販"},
    {"贪", "貪"},
    {"贫", "貧"},
    {"贬", "貶"},
    {"购", "購"},
    {"贮", "貯"},
    {"贯", "貫"},
    {"贰", "貳"},
    {"贱", "賤"},
    {"贲", "賁"},
    {"贳", "貰"},
    {"贴", "貼"},
    {"贵", "貴"},
    {"贶", "貺"},
    {"贷", "貸"},
    {"贸", "貿"},
    {"费", "費"},
    {"贺", "賀"},
    {"贻", "貽"},
    {"贼", "賊"},
    {"贽", "贄"},
    {"贾", "賈"},
    {"贿", "賄"},
    {"赀", "貲"},
    {"赁", "賃"},
    {"赂", "賂"},
    {"赃", "贓"},
    {"资", "資"},
    {"赅", "賅"},
    {"赆", "贐"},
    {"赇", "賕"},
    {"赈", "賑"},
    {"赉", "賚"},
    {"赊", "賒"},
    {"赋", "賦"},
    {"赌", "賭"},
    {"赍", "齎"},
    {"赎", "贖"},
    {"赏", "賞"},
    {"赐", "賜"},
    {"赓", "賡"},
    {"赔", "賠"},
    {"赕", "賧"},
    {"赖", "賴"},
    {"赘", "贅"},
    {"赙", "賻"},
    {"赚", "賺"},
    {"赛", "賽"},
    {"赜", "賾"},
    {"赝", "贗"},
    {"赞", "贊"},
    {"赠", "贈"},
    {"赡", "贍"},
    {"赢", "贏"},
    {"赣", "贛"},
    {"赵", "趙"},
    {"赶", "趕"},
    {"趋", "趨"},
    {"趱", "趲"},
    {"趸", "躉"},
    {"跃", "躍"},
    {"跄", "蹌"},
    {"跞", "躒"},
    {"践", "踐"},
    {"跷", "蹺"},
    {"跸", "蹕"},
    {"跹", "躚"},
    {"跻", "躋"},
    {"踊", "踴"},
    {"踌", "躊"},
    {"踪", "蹤"},
    {"踬", "躓"},
    {"踯", "躑"},
    {"蹑", "躡"},
    {"蹒", "蹣"},
    {"蹰", "躕"},
    {"蹿", "躥"},
    {"躏", "躪"},
    {"躜", "躦"},
    {"躯", "軀"},
    {"车", "車"},
    {"轧", "軋"},
    {"轨", "軌"},
    {"轩", "軒"},
    {"轫", "軔"},
    {"转", "轉"},
    {"轭", "軛"},
    {"轮", "輪"},
    {"软", "軟"},
    {"轰", "轟"},
    {"轲", "軻"},
    {"轳", "轤"},
    {"轴", "軸"},
    {"轵", "軹"},
    {"轶", "軼"},
    {"轸", "軫"},
    {"轹", "轢"},
    {"轺", "軺"},
    {"轻", "輕"},
    {"轼", "軾"},
    {"载", "載"},
    {"轾", "輊"},
    {"轿", "轎"},
    {"辁", "輇"},
    {"辂", "輅"},
    {"较", "較"},
    {"辄", "輒"},
    {"辅", "輔"},
    {"辆", "輛"},
    {"辇", "輦"},
    {"辈", "輩"},
    {"辉", "輝"},
    {"辊", "輥"},
    {"辋", "輞"},
    {"辍", "輟"},
    {"辎", "輜"},
    {"辏", "輳"},
    {"辐", "輻"},
    {"辑", "輯"},
    {"输", "輸"},
    {"辔", "轡"},
    {"辕", "轅"},
    {"辖", "轄"},
    {"辗", "輾"},
    {"辘", "轆"},
    {"辙", "轍"},
    {"辚", "轔"},
    {"辞", "辭"},
    {"辟", "闢"},
    {"辩", "辯"},
    {"辫", "辮"},
    {"边", "邊"},
    {"辽", "遼"},
    {"达", "達"},
    {"迁", "遷"},
    {"过", "過"},
    {"迈", "邁"},
    {"运", "運"},
    {"还", "還"},
    {"这", "這"},
    {"进", "進"},
    {"远", "遠"},
    {"违", "違"},
    {"连", "連"},
    {"迟", "遲"},
    {"迩", "邇"},
    {"迳", "逕"},
    {"迹", "蹟"},
    {"适", "適"},
    {"选", "選"},
    {"逊", "遜"},
    {"递", "遞"},
    {"逦", "邐"},
    {"逻", "邏"},
    {"遗", "遺"},
    {"遥", "遙"},
    {"邓", "鄧"},
    {"邝", "鄺"},
    {"邬", "鄔"},
    {"邮", "郵"},
    {"邹", "鄒"},
    {"邺", "鄴"},
    {"邻", "鄰"},
    {"郁", "鬱"},
    {"郄", "隙"},
    {"郏", "郟"},
    {"郐", "鄶"},
    {"郑", "鄭"},
    {"郓", "鄆"},
    {"郦", "酈"},
    {"郧", "鄖"},
    {"郸", "鄲"},
    {"酝", "醞"},
    {"酱", "醬"},
    {"酽", "釅"},
    {"酾", "釃"},
    {"酿", "釀"},
    {"采", "採"},
    {"释", "釋"},
    {"里", "裡"},
    {"鉴", "鑑"},
    {"銮", "鑾"},
    {"錾", "鏨"},
    {"钅", "金"},
    {"钆", "釓"},
    {"钇", "釔"},
    {"针", "針"},
    {"钉", "釘"},
    {"钊", "釗"},
    {"钋", "釙"},
    {"钌", "釕"},
    {"钍", "釷"},
    {"钎", "釬"},
    {"钏", "釧"},
    {"钐", "釤"},
    {"钒", "釩"},
    {"钓", "釣"},
    {"钔", "鍆"},
    {"钕", "釹"},
    {"钗", "釵"},
    {"钙", "鈣"},
    {"钛", "鈦"},
    {"钜", "鉅"},
    {"钝", "鈍"},
    {"钞", "鈔"},
    {"钟", "鐘"},
    {"钠", "鈉"},
    {"钡", "鋇"},
    {"钢", "鋼"},
    {"钣", "鈑"},
    {"钤", "鈐"},
    {"钥", "鑰"},
    {"钦", "欽"},
    {"钧", "鈞"},
    {"钨", "鎢"},
    {"钩", "鉤"},
    {"钪", "鈧"},
    {"钫", "鈁"},
    {"钬", "鈥"},
    {"钭", "鈄"},
    {"钮", "鈕"},
    {"钯", "鈀"},
    {"钰", "鈺"},
    {"钱", "錢"},
    {"钲", "鉦"},
    {"钳", "鉗"},
    {"钴", "鈷"},
    {"钵", "缽"},
    {"钶", "鈳"},
    {"钸", "鈽"},
    {"钹", "鈸"},
    {"钺", "鉞"},
    {"钻", "鑽"},
    {"钼", "鉬"},
    {"钽", "鉭"},
    {"钾", "鉀"},
    {"钿", "鈿"},
    {"铀", "鈾"},
    {"铁", "鐵"},
    {"铂", "鉑"},
    {"铃", "鈴"},
    {"铄", "鑠"},
    {"铅", "鉛"},
    {"铆", "鉚"},
    {"铈", "鈰"},
    {"铉", "鉉"},
    {"铊", "鉈"},
    {"铋", "鉍"},
    {"铌", "鈮"},
    {"铍", "鈹"},
    {"铎", "鐸"},
    {"铐", "銬"},
    {"铑", "銠"},
    {"铒", "鉺"},
    {"铕", "銪"},
    {"铖", "鋮"},
    {"铗", "鋏"},
    {"铙", "鐃"},
    {"铛", "鐺"},
    {"铜", "銅"},
    {"铝", "鋁"},
    {"铟", "銦"},
    {"铠", "鎧"},
    {"铡", "鍘"},
    {"铢", "銖"},
    {"铣", "銑"},
    {"铤", "鋌"},
    {"铥", "銩"},
    {"铧", "鏵"},
    {"铨", "銓"},
    {"铩", "鎩"},
    {"铪", "鉿"},
    {"铫", "銚"},
    {"铬", "鉻"},
    {"铭", "銘"},
    {"铮", "錚"},
    {"铯", "銫"},
    {"铰", "鉸"},
    {"铱", "銥"},
    {"铲", "鏟"},
    {"铳", "銃"},
    {"铴", "鐋"},
    {"铵", "銨"},
    {"银", "銀"},
    {"铷", "銣"},
    {"铸", "鑄"},
    {"铹", "鐒"},
    {"铺", "鋪"},
    {"铼", "錸"},
    {"铽", "鋱"},
    {"链", "鏈"},
    {"铿", "鏗"},
    {"销", "銷"},
    {"锁", "鎖"},
    {"锂", "鋰"},
    {"锄", "鋤"},
    {"锅", "鍋"},
    {"锆", "鋯"},
    {"锇", "鋨"},
    {"锈", "鏽"},
    {"锉", "銼"},
    {"锊", "鋝"},
    {"锋", "鋒"},
    {"锌", "鋅"},
    {"锐", "銳"},
    {"锑", "銻"},
    {"锒", "鋃"},
    {"锓", "鋟"},
    {"锔", "鋦"},
    {"锕", "錒"},
    {"锖", "錆"},
    {"锗", "鍺"},
    {"错", "錯"},
    {"锚", "錨"},
    {"锛", "錛"},
    {"锞", "錁"},
    {"锟", "錕"},
    {"锡", "錫"},
    {"锢", "錮"},
    {"锣", "鑼"},
    {"锤", "鎚"},
    {"锥", "錐"},
    {"锦", "錦"},
    {"锩", "錈"},
    {"锬", "錟"},
    {"锭", "錠"},
    {"键", "鍵"},
    {"锯", "鋸"},
    {"锰", "錳"},
    {"锱", "錙"},
    {"锲", "鍥"},
    {"锴", "鍇"},
    {"锵", "鏘"},
    {"锶", "鍶"},
    {"锷", "鍔"},
    {"锸", "鍤"},
    {"锹", "鍬"},
    {"锺", "鍾"},
    {"锻", "鍛"},
    {"锼", "鎪"},
    {"锾", "鍰"},
    {"镀", "鍍"},
    {"镁", "鎂"},
    {"镂", "鏤"},
    {"镄", "鐨"},
    {"镆", "鏌"},
    {"镇", "鎮"},
    {"镉", "鎘"},
    {"镊", "鑷"},
    {"镌", "鐫"},
    {"镍", "鎳"},
    {"镏", "鎦"},
    {"镐", "鎬"},
    {"镑", "鎊"},
    {"镒", "鎰"},
    {"镓", "鎵"},
    {"镔", "鑌"},
    {"镖", "鏢"},
    {"镗", "鏜"},
    {"镘", "鏝"},
    {"镙", "鏍"},
    {"镛", "鏞"},
    {"镜", "鏡"},
    {"镝", "鏑"},
    {"镞", "鏃"},
    {"镟", "鏇"},
    {"镡", "鐔"},
    {"镣", "鐐"},
    {"镤", "鏷"},
    {"镦", "鐓"},
    {"镧", "鑭"},
    {"镨", "鐠"},
    {"镪", "鏹"},
    {"镫", "鐙"},
    {"镬", "鑊"},
    {"镭", "鐳"},
    {"镯", "鐲"},
    {"镰", "鐮"},
    {"镱", "鐿"},
    {"镳", "鑣"},
    {"镶", "鑲"},
    {"长", "長"},
    {"门", "門"},
    {"闩", "閂"},
    {"闪", "閃"},
    {"闫", "閆"},
    {"闭", "閉"},
    {"问", "問"},
    {"闯", "闖"},
    {"闰", "閏"},
    {"闱", "闈"},
    {"闲", "閒"},
    {"闳", "閎"},
    {"间", "間"},
    {"闵", "閔"},
    {"闶", "閌"},
    {"闷", "悶"},
    {"闸", "閘"},
    {"闹", "鬧"},
    {"闺", "閨"},
    {"闻", "聞"},
    {"闼", "闥"},
    {"闽", "閩"},
    {"闾", "閭"},
    {"阀", "閥"},
    {"阁", "閣"},
    {"阂", "閡"},
    {"阃", "閫"},
    {"阄", "鬮"},
    {"阅", "閱"},
    {"阆", "閬"},
    {"阈", "閾"},
    {"阉", "閹"},
    {"阊", "閶"},
    {"阋", "鬩"},
    {"阌", "閿"},
    {"阍", "閽"},
    {"阎", "閻"},
    {"阏", "閼"},
    {"阐", "闡"},
    {"阑", "闌"},
    {"阒", "闃"},
    {"阔", "闊"},
    {"阕", "闋"},
    {"阖", "闔"},
    {"阗", "闐"},
    {"阙", "闕"},
    {"阚", "闞"},
    {"阝", "阜"},
    {"队", "隊"},
    {"阳", "陽"},
    {"阴", "陰"},
    {"阵", "陣"},
    {"阶", "階"},
    {"际", "際"},
    {"陆", "陸"},
    {"陇", "隴"},
    {"陈", "陳"},
    {"陉", "陘"},
    {"陕", "陝"},
    {"陧", "隉"},
    {"陨", "隕"},
    {"险", "險"},
    {"随", "隨"},
    {"隐", "隱"},
    {"隶", "隸"},
    {"隽", "雋"},
    {"难", "難"},
    {"雏", "雛"},
    {"雠", "讎"},
    {"雳", "靂"},
    {"雾", "霧"},
    {"霁", "霽"},
    {"霭", "靄"},
    {"靓", "靚"},
    {"静", "靜"},
    {"靥", "靨"},
    {"鞑", "韃"},
    {"鞒", "橇"},
    {"鞯", "韉"},
    {"韦", "韋"},
    {"韧", "韌"},
    {"韩", "韓"},
    {"韪", "韙"},
    {"韫", "韞"},
    {"韬", "韜"},
    {"韵", "韻"},
    {"页", "頁"},
    {"顶", "頂"},
    {"顷", "頃"},
    {"顸", "頇"},
    {"项", "項"},
    {"顺", "順"},
    {"须", "須"},
    {"顼", "頊"},
    {"顽", "頑"},
    {"顾", "顧"},
    {"顿", "頓"},
    {"颀", "頎"},
    {"颁", "頒"},
    {"颂", "頌"},
    {"颃", "頏"},
    {"预", "預"},
    {"颅", "顱"},
    {"领", "領"},
    {"颇", "頗"},
    {"颈", "頸"},
    {"颉", "頡"},
    {"颊", "頰"},
    {"颌", "頜"},
    {"颍", "潁"},
    {"颏", "頦"},
    {"颐", "頤"},
    {"频", "頻"},
    {"颓", "頹"},
    {"颔", "頷"},
    {"颖", "穎"},
    {"颗", "顆"},
    {"题", "題"},
    {"颚", "顎"},
    {"颛", "顓"},
    {"颜", "顏"},
    {"额", "額"},
    {"颞", "顳"},
    {"颟", "顢"},
    {"颠", "顛"},
    {"颡", "顙"},
    {"颢", "顥"},
    {"颤", "顫"},
    {"颦", "顰"},
    {"颧", "顴"},
    {"风", "風"},
    {"飑", "颮"},
    {"飒", "颯"},
    {"飓", "颶"},
    {"飕", "颼"},
    {"飘", "飄"},
    {"飙", "飆"},
    {"飞", "飛"},
    {"飨", "饗"},
    {"餍", "饜"},
    {"饣", "食"},
    {"饥", "飢"},
    {"饧", "餳"},
    {"饨", "飩"},
    {"饩", "餼"},
    {"饪", "飪"},
    {"饫", "飫"},
    {"饬", "飭"},
    {"饭", "飯"},
    {"饮", "飲"},
    {"饯", "餞"},
    {"饰", "飾"},
    {"饱", "飽"},
    {"饲", "飼"},
    {"饴", "飴"},
    {"饵", "餌"},
    {"饶", "饒"},
    {"饷", "餉"},
    {"饺", "餃"},
    {"饼", "餅"},
    {"饽", "餑"},
    {"饿", "餓"},
    {"馀", "餘"},
    {"馁", "餒"},
    {"馄", "餛"},
    {"馅", "餡"},
    {"馆", "館"},
    {"馈", "饋"},
    {"馊", "餿"},
    {"馋", "饞"},
    {"馍", "糢"},
    {"馏", "餾"},
    {"馐", "饈"},
    {"馑", "饉"},
    {"馒", "饅"},
    {"馔", "饌"},
    {"马", "馬"},
    {"驭", "馭"},
    {"驮", "馱"},
    {"驯", "馴"},
    {"驰", "馳"},
    {"驱", "驅"},
    {"驳", "駁"},
    {"驴", "驢"},
    {"驵", "駔"},
    {"驶", "駛"},
    {"驷", "駟"},
    {"驸", "駙"},
    {"驹", "駒"},
    {"驺", "騶"},
    {"驻", "駐"},
    {"驼", "駝"},
    {"驽", "駑"},
    {"驾", "駕"},
    {"驿", "驛"},
    {"骀", "駘"},
    {"骁", "驍"},
    {"骂", "罵"},
    {"骄", "驕"},
    {"骅", "驊"},
    {"骆", "駱"},
    {"骇", "駭"},
    {"骈", "駢"},
    {"骊", "驪"},
    {"骋", "騁"},
    {"验", "驗"},
    {"骏", "駿"},
    {"骐", "騏"},
    {"骑", "騎"},
    {"骒", "騍"},
    {"骓", "騅"},
    {"骖", "驂"},
    {"骗", "騙"},
    {"骘", "騭"},
    {"骚", "騷"},
    {"骛", "騖"},
    {"骜", "驁"},
    {"骝", "騮"},
    {"骞", "騫"},
    {"骟", "騸"},
    {"骠", "驃"},
    {"骡", "騾"},
    {"骢", "驄"},
    {"骣", "驏"},
    {"骤", "驟"},
    {"骥", "驥"},
    {"骧", "驤"},
    {"髅", "髏"},
    {"髋", "髖"},
    {"髌", "髕"},
    {"鬓", "鬢"},
    {"魇", "魘"},
    {"魉", "魎"},
    {"鱼", "魚"},
    {"鱿", "魷"},
    {"鲁", "魯"},
    {"鲂", "魴"},
    {"鲇", "鯰"},
    {"鲈", "鱸"},
    {"鲋", "鮒"},
    {"鲍", "鮑"},
    {"鲎", "鱟"},
    {"鲐", "鮐"},
    {"鲑", "鮭"},
    {"鲒", "鮚"},
    {"鲔", "鮪"},
    {"鲕", "鮞"},
    {"鲚", "鱭"},
    {"鲛", "鮫"},
    {"鲜", "鮮"},
    {"鲞", "鯗"},
    {"鲟", "鱘"},
    {"鲠", "鯁"},
    {"鲡", "鱺"},
    {"鲢", "鰱"},
    {"鲣", "鰹"},
    {"鲤", "鯉"},
    {"鲥", "鰣"},
    {"鲦", "鰷"},
    {"鲧", "鯀"},
    {"鲨", "鯊"},
    {"鲩", "鯇"},
    {"鲫", "鯽"},
    {"鲭", "鯖"},
    {"鲮", "鯪"},
    {"鲰", "鯫"},
    {"鲱", "鯡"},
    {"鲲", "鯤"},
    {"鲳", "鯧"},
    {"鲵", "鯢"},
    {"鲶", "鯰"},
    {"鲷", "鯛"},
    {"鲸", "鯨"},
    {"鲻", "鯔"},
    {"鲽", "鰈"},
    {"鳃", "鰓"},
    {"鳄", "鱷"},
    {"鳅", "鰍"},
    {"鳆", "鰒"},
    {"鳇", "鰉"},
    {"鳌", "鰲"},
    {"鳍", "鰭"},
    {"鳎", "鰨"},
    {"鳏", "鰥"},
    {"鳐", "鰩"},
    {"鳓", "鰳"},
    {"鳔", "鰾"},
    {"鳕", "鱈"},
    {"鳖", "鱉"},
    {"鳗", "鰻"},
    {"鳜", "鱖"},
    {"鳝", "鱔"},
    {"鳞", "鱗"},
    {"鳟", "鱒"},
    {"鳢", "鱧"},
    {"鸟", "鳥"},
    {"鸠", "鳩"},
    {"鸡", "雞"},
    {"鸢", "鳶"},
    {"鸣", "鳴"},
    {"鸥", "鷗"},
    {"鸦", "鴉"},
    {"鸨", "鴇"},
    {"鸩", "鴆"},
    {"鸪", "鴣"},
    {"鸫", "鶇"},
    {"鸬", "鸕"},
    {"鸭", "鴨"},
    {"鸯", "鴦"},
    {"鸱", "鴟"},
    {"鸲", "鴝"},
    {"鸳", "鴛"},
    {"鸵", "鴕"},
    {"鸶", "鷥"},
    {"鸷", "鷙"},
    {"鸸", "鴯"},
    {"鸹", "鴰"},
    {"鸺", "鵂"},
    {"鸽", "鴿"},
    {"鸾", "鸞"},
    {"鸿", "鴻"},
    {"鹁", "鵓"},
    {"鹂", "鸝"},
    {"鹃", "鵑"},
    {"鹄", "鵠"},
    {"鹅", "鵝"},
    {"鹆", "鵒"},
    {"鹇", "鷴"},
    {"鹈", "鵜"},
    {"鹉", "鵡"},
    {"鹊", "鵲"},
    {"鹌", "鵪"},
    {"鹎", "鵯"},
    {"鹏", "鵬"},
    {"鹑", "鶉"},
    {"鹕", "鶘"},
    {"鹗", "鶚"},
    {"鹘", "鶻"},
    {"鹚", "鶿"},
    {"鹜", "鶩"},
    {"鹞", "鷂"},
    {"鹣", "鶼"},
    {"鹤", "鶴"},
    {"鹦", "鸚"},
    {"鹧", "鷓"},
    {"鹨", "鷚"},
    {"鹩", "鷯"},
    {"鹪", "鷦"},
    {"鹫", "鷲"},
    {"鹬", "鷸"},
    {"鹭", "鷺"},
    {"鹰", "鷹"},
    {"鹳", "鸛"},
    {"鹾", "鹺"},
    {"麦", "麥"},
    {"麸", "麩"},
    {"麽", "麼"},
    {"黄", "黃"},
    {"黉", "黌"},
    {"黩", "黷"},
    {"黪", "黲"},
    {"黾", "黽"},
    {"鼋", "黿"},
    {"鼍", "鼉"},
    {"鼹", "鼴"},
    {"齐", "齊"},
    {"齑", "齏"},
    {"齿", "齒"},
    {"龀", "齔"},
    {"龃", "齟"},
    {"龄", "齡"},
    {"龅", "齙"},
    {"龆", "齠"},
    {"龇", "齜"},
    {"龈", "齦"},
    {"龉", "齬"},
    {"龊", "齪"},
    {"龋", "齲"},
    {"龌", "齷"},
    {"龙", "龍"},
    {"龚", "龔"},
    {"龛", "龕"},
    {"龟", "龜"},
    {"＂", "〞"},
    {"［", "﹝"},
    {"］", "﹞"},
    {"｀", "‵"}
};

static const ChineseCharMaps maps[] =
{
    {big5_elements, (sizeof(big5_elements) / sizeof(big5_elements[0])-1)},
    {gb2312_elements, (sizeof(gb2312_elements) / sizeof(gb2312_elements[0])-1)}
};

static int convertChineseChar(const char *utf8, char *catstring, int idx)
{
    int low, mid, high;

    low = 0;
    high = maps[idx].total;

    // 二分搜尋
    while (low <= high)
    {
	mid = (low + high) >> 1;

        int cmplen = strlen(maps[idx].elements[mid].source);
	int chk = strncmp(maps[idx].elements[mid].source, utf8, cmplen);

	if (chk == 0) // 找到了
	{
	    strcat(catstring, maps[idx].elements[mid].target);
	    return 1;
	}

        if (chk > 0)
	{
	    high = mid -1;
	}
	else
	{
	    low = mid + 1;
	}
    }
    return 0;
}

static int b2g(const char *utf8, char *catstring)
{
    return convertChineseChar(utf8, catstring, 0);
}

static int g2b(const char *utf8, char *catstring)
{
    return convertChineseChar(utf8, catstring, 1);
}

char *oxim_output_filter(char *utf8, int isb2g)
{
    unsigned int utf8len = strlen(utf8);
    unsigned int need = ((utf8len / 3 ) * 4 ) + 1; // 最大到 4 bytes utf8
    if (need == 1) need += 4;

    char *work = (char *)oxim_malloc(need, 1);
    unsigned int i=0;
    char *p = utf8;

    while (i < utf8len)
    {
	unsigned char c = p[0];
	unsigned int bytes = 0;

	if ((c & 0xf0) == 0xf0)
	    bytes = 4;
	else if ((c & 0xe0) == 0xe0)
	    bytes = 3;
	else if ((c & 0xc0) == 0xc0)
	    bytes = 2;
        else
	    bytes = 1;

	if (bytes == 1 || !(isb2g ? b2g(p, work) : g2b(p, work)))
	{
	    strncat(work, p, bytes);
	}
	p += bytes;
	i += bytes;
    }
    return work;
}
