/*
    Copyright (C) 2006 by OXIM TEAM

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <X11/Xos.h>
#include "oximtool.h"
#include "gui.h"
#include "oxim.h"

#include "images/en.xpm"
#include "images/ch.xpm"
int check_tray_manager(void);

static int  tray_init = False;
static char **icon = NULL;
static Window manager_window = None;

#define SYSTEM_TRAY_REQUEST_DOCK    0
#define SYSTEM_TRAY_BEGIN_MESSAGE   1
#define SYSTEM_TRAY_CANCEL_MESSAGE  2

static void send_message(
     Display* dpy, /* display */
     Window win,     /* sender (tray icon window) */
     long message, /* message opcode */
     long data1,   /* message data 1 */
     long data2,   /* message data 2 */
     long data3    /* message data 3 */
){
    XEvent ev;
  
    bzero(&ev, sizeof(ev));
    ev.xclient.type = ClientMessage;
    ev.xclient.window = win;
    ev.xclient.message_type = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", False );
    ev.xclient.format = 32;
    ev.xclient.data.l[0] = CurrentTime;
    ev.xclient.data.l[1] = message;
    ev.xclient.data.l[2] = data1;
    ev.xclient.data.l[3] = data2;
    ev.xclient.data.l[4] = data3;

//    trap_errors();
    XSendEvent(dpy, win, False, NoEventMask, &ev);
    XSync(dpy, False);
/*
    if (untrap_errors()) {
    }
*/
}

static void gui_tray_draw(winlist_t *win)
{
    xccore_t *xccore = (xccore_t *)win->data;
    IC *ic = xccore->ic;
    if (win->winmode & WMODE_EXIT || !tray_init)
	return;

    inp_state_t inp_state = (ic) ? ic->imc->inp_state : 0;
    char **disp_icon = (inp_state & IM_CINPUT) ? icon_ch : icon_en;
    if (disp_icon == icon)
	return;

    XWindowAttributes attr;
    XGetWindowAttributes(gui->display, win->window, &attr);
    win->width = attr.width;
    win->height = attr.height;
    int x = (attr.width - 20) / 2;
    int y = (attr.height - 20) / 2;

    gui_Draw_Image(win, 1, 1, disp_icon);
    gui_Draw_3D_Box(win, 0, 0, win->width-1, win->height-1, NO_COLOR, 1);
}

static void gui_focus_event(winlist_t *win, XEvent *event)
{
    switch (event->type)
    {
	case MotionNotify: /* Mouse Over */
	    break;
	case LeaveNotify: /* Mouse Out */
	    break;
	case Expose:
	    if (event->xexpose.count == 0)
	    {
		gui_tray_draw(win);
	    }
	    break;
	case ButtonPress: /* 按下滑鼠按鍵 */
	    if (event->xbutton.button == Button1)
	    {
		if (gui_menu_actived())
		{
		    gui_hide_menu();
		}
		else
		{
		    gui_show_menu(win);
		}
	    }
	    break;
    }
}

/*
    建立 Tray 
*/
void gui_tray_init(void)
{
    winlist_t *win = NULL;

    if (gui->tray_win)
    {
	win = gui->tray_win;
    }
    else
    {
	win = gui_new_win();
    }

    /* 邊框顏色 */
    XSetWindowBorder(gui->display, win->window, gui_Color(WINBOX_COLOR));
    /* 背景顏色 */
    XSetWindowBackground(gui->display, win->window, gui_Color(BORDER_COLOR));

    win->width  = 24;
    win->height = 24;
    win->pos_x = 0;
    win->pos_y = 0;

    win->win_draw_func = gui_tray_draw;
    win->win_event_func = gui_focus_event;

    XSizeHints *size_hints = XAllocSizeHints();
    size_hints->flags = PMinSize;
    size_hints->min_width = win->width;
    size_hints->min_height = win->height;
    XSetWMNormalHints(gui->display, win->window, size_hints);
    XFree(size_hints);

    Atom xembed_info = XInternAtom(gui->display, "_XEMBED_INFO", False);
    unsigned int mapping[2];
    mapping[0] = 0; /* ver 0 */
    mapping[1] = 1; /* request mapping */
    XChangeProperty(gui->display, win->window, xembed_info, xembed_info,
			32, PropModeReplace, (unsigned char *)mapping, 2);
    gui->tray_win = win;
}

int check_tray_manager(void)
{
    if (tray_init)
	return True;

    char tray_mgr[32];
    sprintf(tray_mgr, "_NET_SYSTEM_TRAY_S%d", gui->screen);
    Atom sys_tray = XInternAtom(gui->display, tray_mgr, False);
    if (sys_tray == None)
    {
	return False;
    }

    XGrabServer(gui->display);
    manager_window = XGetSelectionOwner(gui->display, sys_tray);

    if (manager_window != None)
    {
	XSelectInput(gui->display, manager_window, StructureNotifyMask);
	send_message(gui->display, manager_window, SYSTEM_TRAY_REQUEST_DOCK, gui->tray_win->window, 0, 0);
	tray_init = True;
    }
    XUngrabServer(gui->display);
    XFlush(gui->display);

    return tray_init;
}
